package org.paw.filter;


import java.util.StringTokenizer;
import sunlabs.brazil.server.Request;
import sunlabs.brazil.server.Server;
import sunlabs.brazil.util.http.MimeHeaders;
import sunlabs.brazil.filter.Filter;
import sunlabs.brazil.util.regexp.Regexp;

import java.io.*;
import java.util.*;
import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;


public class AnimatedGifFilter implements Filter {
    String prefix;
 
    public boolean init(Server server, String prefix) {
        this.prefix = prefix;

        return true;
    }

    /**
     * This is the request object before the content was fetched
     */

    public boolean respond(Request request) {
        return false;
    }

    /**
     * Only filter image/gif documents
     */

    public boolean shouldFilter(Request request, MimeHeaders headers) {
        String type = headers.get("content-type");

        return (type != null && type.startsWith("image/gif"));
    }

    /**
     * Wrap html around text/plain, converting it to html.
     * Change the content-type to text/html.
     */

    public byte[] filter(Request request, MimeHeaders headers, byte[] content) {
        return (checkImage(content));
    }

    /**
     */

    private byte[] checkImage(byte[] image) {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        DataOutputStream dos = new DataOutputStream(bos);

        try {
            ByteArrayInputStream is = new ByteArrayInputStream(image);

            String s = null;

            // GET SIGNATURE
            byte gifSignature[] = new byte[6];

            is.read(gifSignature);
            dos.write(gifSignature);

            if (!new String(gifSignature).equals("GIF89a"))
                return image;
         
            // Now we read over the SCREEN DESCRIPTOR
            byte singleByte[] = new byte[1];
            byte twoBytes[] = new byte[2];

            // Read GLOBAL COLORMAP
            is.read(twoBytes); 
            dos.write(twoBytes);

            is.read(twoBytes); 
            dos.write(twoBytes);

            byte screenFlags[] = new byte[1];

            is.read(screenFlags); 
            dos.write(screenFlags);

            is.read(twoBytes); 
            dos.write(twoBytes);

            int intScreenFlags = (int) (screenFlags[0] & 0x0ff);

            // If there is a global color table
            if ((intScreenFlags & 128) != 0) {
                int depth = (intScreenFlags & 7) + 1; // Bits per pixel
                int globalColorMapSize = (int) (3 << depth); // Size of colormap
                // Read the colormap
                byte globalColorMap[] = new byte[globalColorMapSize];

                is.read(globalColorMap);
                dos.write(globalColorMap);
            }

            // Check for GIF Extension blocks
            is.read(singleByte);
            dos.write(singleByte); 

            //while((int)singleByte[0] != 33 && (int)singleByte[0] != 44)
            while (true) {
                if ((int) singleByte[0] == 44)
                    break;
 
                if ((int) singleByte[0] == 33) {
                    // Read function code
                    is.read(singleByte); 
                    dos.write(singleByte);

                    while (true) {
                        // Read byte count   
                        byte byteCount[] = new byte[1];

                        is.read(byteCount); 
                        dos.write(byteCount);

                        // Exit loop if extension block is finished
                        if ((int) byteCount[0] == 0)
                            break;

                        int intByteCount = (int) (byteCount[0] & 0x0ff);

                        byte extBytes[] = new byte[intByteCount];

                        is.read(extBytes);
                        dos.write(extBytes);
                    }
                }

                is.read(singleByte);
                dos.write(singleByte);
            }
          
            // Now we read till we find an IMAGE DESCRIPTOR
            while (!new String(singleByte).equals(",")) {
                is.read(singleByte);
                dos.write(singleByte);
            }
         
            // Read the IMAGE DESCRIPTOR
            byte byteX0[] = new byte[2];
            byte byteY0[] = new byte[2];
            byte byteX1[] = new byte[2];
            byte byteY1[] = new byte[2];

            is.read(byteX0);
            is.read(byteY0);
            is.read(byteX1);
            is.read(byteY1);

            dos.write(byteX0);
            dos.write(byteY0);
            dos.write(byteX1);
            dos.write(byteY1);

            int x0 = convTwoBytesToInt(byteX0);
            int y0 = convTwoBytesToInt(byteY0);
            int x1 = convTwoBytesToInt(byteX1);
            int y1 = convTwoBytesToInt(byteY1);
         
            // Read flags
            byte flags[] = new byte[1];

            is.read(flags);
            dos.write(flags);

            int intFlags = (int) (flags[0] & 0x0ff);

            int bits;
            int colorMapSize;

            // If there is a local color table
            if ((intFlags & 128) != 0) {
                bits = (intFlags & 7) + 1; // Bits per pixel
                colorMapSize = (int) (3 << bits); // Size of colormap

                // Read the colormap
                byte colorMap[] = new byte[colorMapSize];

                is.read(colorMap);
                dos.write(colorMap);
            }

            is.read(singleByte);
            dos.write(singleByte);

            while (true) {
                // Read IMAGE DATA
                byte dataSize[] = new byte[1];

                is.read(dataSize);
                dos.write(dataSize);

                // Exit loop if image data is finished
                if ((int) dataSize[0] == 0)
                    break;

                int intDataSize = (int) (dataSize[0] & 0x0ff);

                byte imageData[] = new byte[intDataSize];

                is.read(imageData);
                dos.write(imageData);
            }

            //First frame is read, write GIF End Tag
            dos.write(';');
          
        } catch (Exception ex) {
            System.err.println(ex.getMessage());
        }

        byte ret[] = bos.toByteArray();

        try {
            bos.close();
        } catch (Exception ex) {
            System.out.println(ex.getMessage());
        }

        return (ret);
    }

    private static int convTwoBytesToInt(byte[] bytes) {
        byte byteTmp = bytes[1];

        bytes[1] = bytes[0];
        bytes[0] = byteTmp;

        int res =  
            ((bytes[0] << 8) & 0xff00)
            | (((int) bytes[1] & 0x0ff));

        return res;
    }

}

