package org.paw.filter;


import java.util.StringTokenizer;
import sunlabs.brazil.server.Request;
import sunlabs.brazil.server.Server;
import sunlabs.brazil.util.http.MimeHeaders;
import sunlabs.brazil.filter.Filter;
import sunlabs.brazil.util.regexp.Regexp;
import sunlabs.brazil.handler.ResourceHandler;

import java.io.*;
import java.util.*;


public class GifSizeFilter implements Filter {
    String prefix;
    byte[] replaceImage;
    Hashtable sizesHash = new Hashtable();
  
    public boolean init(Server server, String prefix) {
        this.prefix = prefix;
    
        Properties props = server.props;
	
        String sizeFile = props.getProperty(prefix + "filename", "");
        String imageName = props.getProperty(prefix + "image", "");

        try {
            replaceImage = ResourceHandler.getResourceBytes(props, prefix, imageName);	
        } catch (Exception e) {
            server.log(Server.LOG_DIAGNOSTIC, prefix, "error reading image file");
            return false;
        } 
    
        if (!readSizeFile(sizeFile))
            return false;
        
        return true;
    }

    /**
     * This is the request object before the content was fetched
     */

    public boolean respond(Request request) {
        return false;
    }

    /**
     * Only filter image/gif documents
     */

    public boolean shouldFilter(Request request, MimeHeaders headers) {
        String type = headers.get("content-type");

        return (type != null && type.startsWith("image/gif"));
    }

    /**
     * Wrap html around text/plain, converting it to html.
     * Change the content-type to text/html.
     */

    public byte[] filter(Request request, MimeHeaders headers, byte[] content) {
        return (checkImage(content));
    }

    /**
     */

    private byte[] checkImage(byte[] image) {
        try {
            ByteArrayInputStream is = new ByteArrayInputStream(image);

            String s = null;

            // GET SIGNATURE
            byte gifSignature[] = new byte[6];

            is.read(gifSignature);

            /*
             * Scan all images
             if(!new String(gifSignature).equals("GIF89a"))
             return image;
             */
         
            // Now we read over the SCREEN DESCRIPTOR
            byte singleByte[] = new byte[1];
            byte twoBytes[] = new byte[2];

            // Read image size
            is.read(twoBytes);
            int width = convTwoBytesToInt(twoBytes);

            is.read(twoBytes);  
            int height = convTwoBytesToInt(twoBytes);
          
            if (sizesHash.get(width + "x" + height) != null)
                return replaceImage;

        } catch (Exception ex) {}   
         
        return image;
    }

    private static int convTwoBytesToInt(byte[] bytes) {
        byte byteTmp = bytes[1];

        bytes[1] = bytes[0];
        bytes[0] = byteTmp;

        int res =  
            ((bytes[0] << 8) & 0xff00)
            | (((int) bytes[1] & 0x0ff));

        return res;
    }
   
    private boolean readSizeFile(String filename) {
        try {
            File f = new File(filename);
           
            BufferedReader is = new BufferedReader(new FileReader(f));
            String s = null;
            Vector sVect = new Vector();

            while ((s = is.readLine()) != null) {
                if (s.startsWith("#"))
                    continue;

                sizesHash.put(s, "");
            }
            is.close();
        } catch (Exception ex) {
            return false;
        }
        
        return true;
    }

}

