package org.paw.filter;


import java.util.StringTokenizer;
import java.util.zip.*;
import sunlabs.brazil.server.Request;
import sunlabs.brazil.server.Server;
import sunlabs.brazil.util.http.MimeHeaders;
import sunlabs.brazil.filter.Filter;
import sunlabs.brazil.util.regexp.Regexp;

import java.io.*;
import java.util.*;
import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;

import gnu.regexp.*;


public class PawRegFilter implements Filter
{
   String config;
   String prefix;
   Hashtable sizeFileHash = new Hashtable();
   Vector activeReplaceFilters = new Vector();
   Vector activeSizeFilters = new Vector();

   public boolean init(Server server, String prefix)
   {
      this.prefix = prefix;

      // Get the config file property
      Properties props = server.props;

      this.config = props.getProperty(prefix + "config", "");

      try
      {
         // No validation
         SAXBuilder builder = new SAXBuilder(false);
         Document doc = builder.build(new File(config));

         // Get root element
         Element root = doc.getRootElement();

         List filters = root.getChildren("reg-filter");

         // Loop through all filters
         Iterator i = filters.iterator();

         while (i.hasNext())
         {
            Element filter = (Element) i.next();

            PawRegExpFilter pf = new PawRegExpFilter();

            String type = filter.getAttributeValue("type");
            String status = filter.getAttributeValue("status");
            String name = filter.getChild("name").getText();
            String description = filter.getChild("description").getText();

            pf.setType(type);
            pf.setStatus(status);
            pf.setName(name);
            pf.setDescription(description);

            if (type.equals("size"))
            {
               String file = filter.getChild("file").getText();

               pf.setFile(file);
            }

            List exp = filter.getChildren("regexp");
            Iterator e = exp.iterator();

            while (e.hasNext())
            {
               Element regexp = (Element) e.next();
               String from = regexp.getChild("match").getText();
               String to = regexp.getChild("subst").getText();

               pf.addRegExp(from, to);
            }

            // Only include active filters
            if (pf.getStatus().equals("active"))
            {
               if (pf.getType().equals("replace"))
                  activeReplaceFilters.add(pf);
               else
                  activeSizeFilters.add(pf);

               server.log(Server.LOG_INFORMATIONAL, prefix,
                       "Added filter " + name);
            }

         }

         sizeFileHash = getFileHash(activeSizeFilters);

      }
      catch (Exception ex)
      {
         server.log(Server.LOG_ERROR, prefix, ex.getMessage());
         return false;
      }

      return true;
   }

   /**
     * This is the request object before the content was fetched
     */

   public boolean respond(Request request)
   {
      return false;
   }

   /**
     * Only filter text/plain documents
     */

   public boolean shouldFilter(Request request, MimeHeaders headers)
   {
      String type = headers.get("content-type");

      return (type != null && type.startsWith("text/html"));
   }

   /**
     * Wrap html around text/plain, converting it to html.
     * Change the content-type to text/html.
     */

   public byte[] filter(Request request, MimeHeaders headers,
           byte[] content)
   {
      String uncompressHeader = headers.get("X-PAW-DECOMPRESSED");       

      // Do not handle if content ist still packed
      if ( uncompressHeader != null && uncompressHeader.equals("false") )
         return content;


      String result = checkContent(new String(content));



      return result.getBytes();

   }


   /**
     */

   private String checkContent(String content)
   {

      /*
       Some sites mess up with mimetypes
       only inspect content, that has an html tag
       */
      //Regexp regExp = new Regexp("<[ ]*html", true);

      //if (regExp.match(content) == null)
      //return content;

      //First call the replace filters
      for (Enumeration e = activeReplaceFilters.elements();
              e.hasMoreElements();)
      {
         PawRegExpFilter pf = (PawRegExpFilter) e.nextElement();

         Vector v = pf.getRegExp();

         for (Enumeration r = v.elements(); r.hasMoreElements();)
         {
            RE re;
            PawRegExp pr = (PawRegExp) r.nextElement();

            if (pr.getMatch().equals("@START"))
            {
               content = pr.getSubst() + content;
            }
            else if (pr.getMatch().equals("@END"))
            {
               content += pr.getSubst();
            }
            else
            {
              try {
                 re = new RE(pr.getMatch(), RE.REG_ICASE|RE.REG_DOT_NEWLINE);
                 content = re.substituteAll(content, pr.getSubst());
              }
              catch (REException reEx) {
                 reEx.printStackTrace();
              }
            }

         }
      }

      // Call size filters

      for (Enumeration e = activeSizeFilters.elements();
              e.hasMoreElements();)
      {
         PawRegExpFilter pf = (PawRegExpFilter) e.nextElement();
         String filename = pf.getFile();

         Vector v = pf.getRegExp();

         for (Enumeration r = v.elements(); r.hasMoreElements();)
         {
            PawRegExp pr = (PawRegExp) r.nextElement();
            String match = pr.getMatch();
            String subst = pr.getSubst();

            if (sizeFileHash.get(filename) != null)
            {
               Vector ve = (Vector) sizeFileHash.get(filename);

               for (Enumeration en = ve.elements(); en.hasMoreElements();)
               {
                  Regexp re;

                  String entry = (String) en.nextElement();

                  if (entry == null)
                     continue;

                  String cMatch = new String(match);

                  String[] sizes = new String[3];

                  re = new Regexp("([0-9]+)x([0-9]+)", true);

                  if (!re.match(entry, sizes))
                     continue;

                  String width = sizes[1];
                  String height = sizes[2];

                  
                  re = new Regexp("@WIDTH");
                  cMatch = re.subAll(cMatch, width);

                  re = new Regexp("@HEIGHT");
                  cMatch = re.subAll(cMatch, height);

                  try {
                     RE regExp = new RE(cMatch, RE.REG_ICASE|RE.REG_DOT_NEWLINE);
                     content = regExp.substituteAll(content, subst);
                  }
                  catch (REException reEx) {
                     reEx.printStackTrace();
                  }
                  
               }
            }
         }
      }

      return (content);

   }

   private Hashtable getFileHash(Vector vect)
   {
      Hashtable hash = new Hashtable();

      for (Enumeration e = vect.elements(); e.hasMoreElements();)
      {
         PawRegExpFilter pf = (PawRegExpFilter) e.nextElement();

         if (pf.getType().equals("size"))
         {
            String filename = pf.getFile();

            // If filename is already in hashtable continue with next entry
            if (hash.get(filename) != null)
               continue;

            File f = new File(filename);

            if (f.canRead())
            {
               try
               {
                  BufferedReader is = new BufferedReader(new FileReader(f));
                  String s = null;
                  Vector sVect = new Vector();

                  while ((s = is.readLine()) != null)
                  {
                     if (s.startsWith("#"))
                        continue;

                     sVect.add(s);
                  }
                  is.close();

                  hash.put(filename, sVect);
               }
               catch (Exception ex)
               {
                  continue;
               }
            }
         }
      }

      return hash;
   }
}


class PawRegExpFilter
{
   String name, description, file, type, status;
   Vector regexp;

   public PawRegExpFilter()
   {
      name = null;
      description = null;
      file = null;
      type = null;
      status = null;
      regexp = new Vector();
   }

   public void setName(String name)
   {
      this.name = name;
   }

   public String getName()
   {
      return this.name;
   }

   public void setDescription(String description)
   {
      this.description = description;
   }

   public String getDescription()
   {
      return this.description;
   }

   public void setFile(String file)
   {
      this.file = file;
   }

   public String getFile()
   {
      return this.file;
   }

   public void setType(String type)
   {
      this.type = type;
   }

   public String getType()
   {
      return this.type;
   }

   public void setStatus(String status)
   {
      this.status = status;
   }

   public String getStatus()
   {
      return this.status;
   }

   public void addRegExp(String match, String subst)
   {
      PawRegExp pReg = new PawRegExp(match, subst);

      regexp.add(pReg);

   }

   public Vector getRegExp()
   {
      return regexp;
   }

}


class PawRegExp
{
   String match, subst;

   public PawRegExp(String match, String subst)
   {
      this.match = match;
      this.subst = subst;
   }

   public String getMatch()
   {
      return this.match;
   }

   public String getSubst()
   {
      return this.subst;
   }
}


