package org.paw.filter;


import sunlabs.brazil.server.Request;
import sunlabs.brazil.server.Server;
import sunlabs.brazil.util.http.MimeHeaders;
import sunlabs.brazil.filter.Filter;
import sunlabs.brazil.util.regexp.Regexp;
import sunlabs.brazil.util.http.HttpInputStream;
import sunlabs.brazil.handler.ResourceHandler;

import java.io.*;
import java.util.*;
import org.jdom.*;


public class ResponseHeadersFilter implements Filter {
    String prefix;
    private static final String HEADERS = "headers";

    Hashtable headersHash = null;
 
    public boolean init(Server server, String prefix) {
	this.prefix = prefix;

	Properties props = server.props;
	
	try {
	    String headers = props.getProperty(prefix + HEADERS);
            // Create headers hash
            headersHash = loadHeaders(props, prefix, headers);
	} catch (Exception e) {
	    server.log(Server.LOG_DIAGNOSTIC, prefix,
		    "error in headers: " + e.getMessage());
	}
	return true;
    }

    /**
     * This is the request object before the content was fetched
     */

    public boolean respond(Request request) {
        return false;
    }

    public boolean shouldFilter(Request request, MimeHeaders headers) {
         String type = headers.get("content-type");
         return (type != null && type.startsWith("text/html"));
    }

    public byte[] filter(Request request, MimeHeaders headers, byte[] content) {

        // Replace headers
        for(Enumeration enum = headersHash.keys(); enum.hasMoreElements();) {
            String header = (String) enum.nextElement(); 
            String value  = (String) headersHash.get(header);

            if(!value.equals("")) {
                headers.remove(header);
                headers.add(header, value);
            }
            else {
                headers.remove(header);
            }

        }

        return (content);
   
    }

    public Hashtable
    loadHeaders(Properties props, String prefix, String file)
    {
	Hashtable ht = new Hashtable();
	try {
	    HttpInputStream in = new HttpInputStream(
		    ResourceHandler.getResourceStream(props, prefix, file));

	    String line;
	    while ((line = in.readLine()) != null) {


		if ((line.length() == 0) || (line.charAt(0) == '#')) {
		    continue;
		}

                String header = line.substring(0, line.indexOf(":"));
                String value = line.substring(line.indexOf(":") + 1);
                ht.put(header, value);
	    }

	    return ht;
	} catch (Exception e) {
            // Return a valid hashtable in any case
	    return ht;
	}
    }

}
