package org.paw.filter;


import java.util.StringTokenizer;
import sunlabs.brazil.server.Request;
import sunlabs.brazil.server.Server;
import sunlabs.brazil.util.http.MimeHeaders;
import sunlabs.brazil.filter.Filter;
import sunlabs.brazil.util.regexp.Regexp;
import sunlabs.brazil.handler.ResourceHandler;

import java.io.*;
import java.util.*;


public class VirusFilter implements Filter {
    String prefix;
    Hashtable mimetypes = new Hashtable();
    int codeOk = -1,
        codeVirusRemoved = -1,
        codeVirusNotRemoved = -1;
    String cmdLine = null,
        tmpDir = null,
        quarantineDir = null,
        virusMessage = null;
    int maxFileLength = 255;
  
    public boolean init(Server server, String prefix) {
        this.prefix = prefix;
    
        Properties props = server.props;
	
        String confFile = props.getProperty(prefix + "config", "");

        boolean ret = readConfig(confFile);

        return ret;
    }

    /**
     * This is the request object before the content was fetched
     */

    public boolean respond(Request request) {
        return false;
    }

    /**
     * Only filter image/gif documents
     */

    public boolean shouldFilter(Request request, MimeHeaders headers) {
        String type = headers.get("content-type");
    
        // If all mimetypes should be scanned (paranoia)
        if (mimetypes.get("*") != null)
            return true;
    
        boolean ret = mimetypes.get(type) != null ? true : false;
    
        return ret;
    }

    /**
     * Wrap html around text/plain, converting it to html.
     * Change the content-type to text/html.
     */

    public byte[] filter(Request request, MimeHeaders headers, byte[] content) {
        //System.out.println("Checking for virus");
        return (checkForVirus(content));
    }

    private byte[] checkForVirus(byte[] content) {
        try {
            String tmpFileName = "vc" + new Date().getTime();
        
            int length = tmpFileName.length();
        
            if (length > maxFileLength)
                tmpFileName = tmpFileName.substring(length - maxFileLength);
        
            String tmpFullPath = tmpDir + File.separatorChar + tmpFileName;
            String quarantineFullPath = quarantineDir + File.separatorChar +
                tmpFileName;
        
            // Write content to file
            OutputStream outFile = new FileOutputStream(tmpFullPath);

            outFile.write(content);
            outFile.close();
      
            //Insert filename into command line
            String cmd = new String(cmdLine);
            int pos = cmd.indexOf("%f");
        
            if (pos != -1)
                cmd = cmd.substring(0, pos) + tmpFullPath + cmd.substring(pos + 2);
             
            // Start virus scanne      
            Runtime runtime = Runtime.getRuntime();
            //System.out.println("Executing: " + cmd);
            Process process = runtime.exec(cmd);
        
            //Read the output
            BufferedReader outData = new BufferedReader(
                    new InputStreamReader(process.getInputStream()));

            String outLine = outData.readLine();

            while (outLine != null) {
                //System.out.println(outLine);
                outLine = outData.readLine();
            }

            //System.out.println("Waiting for process...");
        
            process.waitFor();
        
            int code = process.exitValue();
            //System.out.println("Return code: " + code);
        
            File f = new File(tmpFullPath);
        
            if (code == codeVirusRemoved) {
                FileInputStream in = new FileInputStream(tmpFullPath);
                ByteArrayOutputStream out = new ByteArrayOutputStream();
                int len = 1024;
                byte[] buffer = new byte[len];

                while ((len = in.read(buffer)) > 0) {
                    out.write(buffer, 0, len);
                }

                content = out.toByteArray();
            } else if (code == codeVirusNotRemoved) {
                // Copy to quarantine area and replace content with virus message
                content = (virusMessage + "(id=" + tmpFileName + ")").getBytes();
           
                File qFile = new File(quarantineFullPath);
            
                f.renameTo(qFile);
            }
        
            // if(f.exists())
            //   f.delete();

        } catch (Exception ex) {
            ex.printStackTrace();
        }
     
        return content;
      
    }
  
    private boolean readConfig(String filename) {
        try {
            FileInputStream is = new FileInputStream(filename);
            Properties props = new Properties();

            props.load(is);
            is.close();
           
            cmdLine = props.getProperty("cmd_line");
            tmpDir = props.getProperty("tmp_dir");
            quarantineDir = props.getProperty("quarantine_dir");
            virusMessage = props.getProperty("virus_message");
           
            codeOk = new Integer(props.getProperty("code_ok")).intValue();
            codeVirusRemoved =
                    new Integer(props.getProperty("code_virus_removed")).intValue();
            codeVirusNotRemoved =
                    new Integer(props.getProperty("code_virus_not_removed")).intValue();
           
            // Read in the mimetypes
            StringTokenizer st = new StringTokenizer(
                    props.getProperty("mimetypes"));

            while (st.hasMoreTokens()) {
                String token = st.nextToken();

                mimetypes.put(token, "");
            }
           
            if (props.getProperty("max_file_length") != null) {
                maxFileLength =    
                        new Integer(props.getProperty("max_file_length")).intValue();
            }
           
        } catch (Exception ex) {
            return false;
        }
        
        return true;
    }

}

