/*
 * AdminConnection.java
 *
 * Created on 21. Juli 2001, 06:19
 */

package org.paw.gui;


/**
 *
 * @author  Standard
 * @version 
 */

import java.net.*;
import java.io.*;
import java.util.*;
import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;

import javax.swing.*;
import javax.swing.tree.*;


public class AdminConnection {
    private String host = null;
    private int port;
    
    private String user = null;
    private String pass = null;
    
    boolean connectionValid = false;
    boolean authValid = false;
    
    private String lastMessage = null;
    
    private GuiServerConfig serverObj = null;
    private GuiAdminConfig adminObj = null;
    private GuiBlockList blockListObj = null;
  
    private Vector files;
    // This hashtable is used to ensure that each file appears only once in list
    private Hashtable uniqFileNames;
    
    private Vector replaceFilters = null;
    private Vector sizeFilters = null;
    private Vector handlers = null;
    private Vector filters = null;

    /** Creates new AdminConnection */
    public AdminConnection() {}

    public void setPeer(String host, int port) {
        this.host = host;
        this.port = port;
        
        validate();
    }
    
    public GuiServerConfig getServerConfig() {
        return serverObj;
    }
    
    public GuiAdminConfig getAdminConfig() {
        return adminObj;
    }
   
    public Vector getHandlers() {
        return handlers;
    }
    
    public Vector getFilters() {
        return filters;
    }
    
    public Vector getFiles() {
        return files;
    }
    
    public Vector getReplaceFilters() {
        return replaceFilters;
    }
    
    public Vector getSizeFilters() {
        return sizeFilters;
    }
    
    public String getHost() {
        return host;
    }
    
    public int getPort() {
        return port;
    }
    
    public String getUser() {
        return user;
    }
    
    public String getPass() {
        return pass;
    }
    
    public String getLastMessage() {
        return lastMessage;
    }
    
    public GuiServerConfig getServerObj() {
        return serverObj;
    }
    
    private void validate() {
        try {
            Socket socket = new Socket(host, port);

            socket.close();
            connectionValid = true;
        } catch (IOException ex) {
            connectionValid = false;
        }
    }
    
    public boolean isValid() {
        return connectionValid;
    }
    
    public void initFiles() {
        files = new Vector();
        uniqFileNames = new Hashtable();
    }
        
    public void checkAuth(String user, String pass) {
        this.user = user;
        this.pass = pass;
        
        try {
            Socket socket = new Socket(host, port);
            
            InputStream in = socket.getInputStream();
            OutputStream out = socket.getOutputStream();
            
            String line;

            line = readLine(in);
            
            out.write(("user " + user + "\r\n").getBytes());
            out.write(("pass " + pass + "\r\n").getBytes());
            
            line = readLine(in);
     
            if (line.startsWith("240"))
                authValid = true;
            else
                authValid = false;
            
            socket.close();
        } catch (IOException ex) {
            System.out.println(ex.getMessage());
            authValid = false;
        }
    }
    
    public boolean doCommand(String command) {
        try {
            Socket socket = new Socket(host, port);
            
            InputStream in = socket.getInputStream();
            OutputStream out = socket.getOutputStream();
            
            String line;

            line = readLine(in);
            
            out.write(("user " + user + "\r\n").getBytes());
            out.write(("pass " + pass + "\r\n").getBytes());
            
            line = readLine(in);
     
            out.write((command + "\r\n").getBytes());
            
            line = readLine(in);
            
            lastMessage = line;
            
            socket.close();
        } catch (IOException ex) {
            System.out.println(ex.getMessage());
            return false;
        }
        
        return true;
    }
    
    private String readLine(InputStream in) throws IOException {
        int c;
        StringBuffer line = new StringBuffer();

        while ((c = in.read()) != -1) {
            if (c == '\r') {// ignore
            } else if (c == '\n') {
                return (line.toString());
            } else {
                line.append((char) c);
            }
        }
        return (null);
    }

    public String getFileFromServer(String fileName) {
             // If nothing is specified, get conf files
            return( getFileFromServer("getconf", fileName) );
   }   

    public String getFileFromServer(String command, String fileName) {
        try {
            Socket socket = new Socket(host, port);
            
            InputStream in = socket.getInputStream();
            OutputStream out = socket.getOutputStream();
            
            StringBuffer fileBuf = new StringBuffer();
            
            String line;

            line = readLine(in);
            
            out.write(("user " + user + "\r\n").getBytes());
            out.write(("pass " + pass + "\r\n").getBytes());
            
            line = readLine(in);
     
            out.write((command + " " + fileName + "\r\n").getBytes());
            
            line = readLine(in);
            
            if (line.startsWith("400")  || line.startsWith("500"))
                return null;
            
            fileBuf.append(line + "\n");
            
            while ((line = readLine(in)) != null) {
                if (line.startsWith("226"))
                    break;

                fileBuf.append(line + "\n");
            }
            
            socket.close();
            
            return (fileBuf.toString());
        } catch (IOException ex) {
            System.out.println(ex.getMessage());
            return null;
        }
    }

   
    public boolean uploadFile(String fileName, String content) {
        boolean ret = false;
       
        try {
            Socket socket = new Socket(host, port);
            
            InputStream in = socket.getInputStream();
            OutputStream out = socket.getOutputStream();
            
            String line;

            line = readLine(in);
            
            out.write(("user " + user + "\r\n").getBytes());
            out.write(("pass " + pass + "\r\n").getBytes());
            
            line = readLine(in);
     
            out.write(("put " + fileName + "\r\n").getBytes());
            
            line = readLine(in);
            
            if (!line.startsWith("200"))
                ret = false;
            else {
                out.write(content.getBytes());
                out.write(("\n.").getBytes());
            
                if (line.startsWith("226"))
                    ret = true;
                else
                    ret = true;
            }
            
            out.close();
            socket.close();
            
            return ret;
        } catch (IOException ex) {
            System.out.println(ex.getMessage());
            return false;
        }
    }
   
    public boolean hasFilterClass(String className) {
        boolean ret = false;
       
        for (Enumeration e = filters.elements(); e.hasMoreElements() &&
            ret == false;) {
            GuiFilter filter = (GuiFilter) e.nextElement();

            ret = filter.hasClass(className);
        }
       
        return ret;
    }
   
    public boolean hasHandlerClass(String className) {
        boolean ret = false;
       
        for (Enumeration e = handlers.elements(); e.hasMoreElements() &&
            ret == false;) {
            GuiHandler handler = (GuiHandler) e.nextElement();

            ret = handler.hasClass(className);
        }
       
        return ret;
    }
   
    public boolean readServerConfig() {
        String xml = getFileFromServer("server.xml");
      
        if (xml == null)
            return false;
       
        buildServerConfig(xml);
        
        return true;
    }
   
    public boolean readHandlerConfig() {
        String xml = getFileFromServer("handler.xml");
       
        if (xml == null)
            return false;
        
        try {
          buildHandlerConfig(xml);
        }
        catch(Exception e) {
	    e.printStackTrace();
	}
       
        return true;   
    }
   
    public boolean readFilterConfig() {
        String xml = getFileFromServer("filter.xml");
       
        if (xml == null)
            return false;
       try {
          buildFilterConfig(xml);
       }
        catch(Exception e) {
	    e.printStackTrace();
	}

        return true;   
    }
   
    public boolean readRegFilterConfig() {
        String xml = getFileFromServer("reg-filter.xml");

        if (xml == null)
            return false;
        
        try {
          buildRegFilterConfig(xml);
	}
        catch(Exception e) {
	    e.printStackTrace();
	}

        return true;
    }

    public void buildRegFilterConfig(String xml) throws Exception{
        //Initialize Vectors
        if(replaceFilters == null) {
           replaceFilters = new Vector();
           sizeFilters = new Vector();
        }

        Vector fileNames = new Vector();

        //try {
            // No validation
            SAXBuilder builder = new SAXBuilder(false);
            Document doc = builder.build(new StringReader(xml));

            // Get root element
            Element root = doc.getRootElement();

            List filters = root.getChildren("reg-filter");

            // Loop through all filters
            Iterator i = filters.iterator();

            while (i.hasNext()) {
                Element filter = (Element) i.next();

                GuiRegExpFilter pf = new GuiRegExpFilter();

                String type = filter.getAttributeValue("type");
                String status = filter.getAttributeValue("status");
                String name = filter.getChild("name").getText();
                String description = filter.getChild("description").getText();

                pf.setType(type);
                pf.setStatus(status);
                pf.setName(name);
                pf.setDescription(description);

                if (type.equals("size")) {
                    String file = filter.getChild("file").getText();

                    pf.setFile(file);
                    fileNames.add(file);
                }

                List exp = filter.getChildren("regexp");
                Iterator e = exp.iterator();

                while (e.hasNext()) {
                    Element regexp = (Element) e.next();
                    String from = regexp.getChild("match").getText();
                    String to = regexp.getChild("subst").getText();

                    pf.addRegExp(from, to);
                }

                if (status.equals("active"))
                    pf.setActive(true);
                else
                    pf.setActive(false);

                if (pf.getType().equals("replace"))
                    replaceFilters.add(pf);
                else
                    sizeFilters.add(pf);
      
            }

            // Process files
            for (Enumeration e = fileNames.elements(); e.hasMoreElements();) {
                String fileName = (String) e.nextElement();

                if (uniqFileNames.get(fileName) != null)
                    continue;
                else
                    uniqFileNames.put(fileName, "");
            
                fileName = fileName.substring(fileName.indexOf("/") + 1);

                String cont = getFileFromServer(fileName);
            
                // File is not there
                if (cont == null)
                    cont = "";
            
                GuiFile file = new GuiFile();

                file.setName(fileName);
                file.setContent(cont);
            
                // Add file Object to files Vector
                files.add(file);
            }

       //} catch (Exception ex) { ex.printStackTrace(); }
    }
 
    public void buildHandlerConfig(String xml) throws Exception {
        //Initialize Vectors
        if(handlers == null) {
            handlers = new Vector(); 
        }
        
        Vector fileNames = new Vector();
       
        //try {
            // No validation
            SAXBuilder builder = new SAXBuilder(false);
            Document doc = builder.build(new StringReader(xml));
        
            // Get root element
            Element root = doc.getRootElement();

            List handlerList = root.getChildren("handler");

            // Loop through all filters
            Iterator i = handlerList.iterator();

            while (i.hasNext()) {
                Element handler = (Element) i.next();
        
                GuiHandler ph = new GuiHandler();
        
                String status = handler.getAttributeValue("status");
                String name = handler.getChild("name").getText();
                String description = handler.getChild("description").getText();
                String removable = handler.getChild("removable").getText();
                String id = handler.getChild("id").getText();
        
                Element filesElement = handler.getChild("files");

                if (filesElement != null) {
                    List fileList = filesElement.getChildren("file");
        
                    Iterator fe = fileList.iterator();

                    while (fe.hasNext()) {
                        Element fileEntry = (Element) fe.next();
                        String filename = fileEntry.getText();

                        fileNames.add(filename);

                        ph.addFile(filename);
                    }
                }
        
                Element params = handler.getChild("params");

                if (params != null) {
                    List paramList = params.getChildren("param");
                    Iterator pe = paramList.iterator();
        
                    while (pe.hasNext()) {    
                        Element paramEntry = (Element) pe.next();
         
                        String param = paramEntry.getAttributeValue("name");
                        String value = paramEntry.getAttributeValue("value");
      
                        ph.addParam(param, value);
                    }
                }

                ph.setStatus(status);
                if (status.equals("active"))
                    ph.setActive(true);
                else
                    ph.setActive(false);
        
                ph.setName(name);
                ph.setDescription(description);
        
                if (removable.equals("true"))
                    ph.setRemovable(true);
                else
                    ph.setRemovable(false);
        
                ph.setId(id);
        
                handlers.add(ph);      
            }
      
            // Process files
            for (Enumeration e = fileNames.elements(); e.hasMoreElements();) {
                String fileName = (String) e.nextElement();

                if (uniqFileNames.get(fileName) != null)
                    continue;
                else
                    uniqFileNames.put(fileName, "");
            
                fileName = fileName.substring(fileName.indexOf("/") + 1);

                String cont = getFileFromServer(fileName);
            
                // File is not there
                if (cont == null)
                    cont = "";
            
                GuiFile file = new GuiFile();

                file.setName(fileName);
                file.setContent(cont);
            
                // Add file Object to files Vector
                files.add(file);
            }
	/*
        } catch (Exception ex) {
            //System.err.println(ex.getMessage());
            ex.printStackTrace();
        }
	*/
    
    }
 
    public void buildFilterConfig(String xml) throws Exception {
        //Initialize Vectors
        if(filters == null) {
            filters = new Vector(); 
        }
        
        Vector fileNames = new Vector();
       
        //try {
            // No validation
            SAXBuilder builder = new SAXBuilder(false);
            Document doc = builder.build(new StringReader(xml));
        
            // Get root element
            Element root = doc.getRootElement();

            List filterList = root.getChildren("filter");

            // Loop through all filters
            Iterator i = filterList.iterator();

            while (i.hasNext()) {
                Element filter = (Element) i.next();
        
                GuiFilter pf = new GuiFilter();
        
                String status = filter.getAttributeValue("status");
                String name = filter.getChild("name").getText();
        
                String description = filter.getChild("description").getText();
                String removable = filter.getChild("removable").getText();
                String id = filter.getChild("id").getText();
        
                Element filesElement = filter.getChild("files");

                if (filesElement != null) {       
                    List fileList = filesElement.getChildren("file");
        
                    Iterator fe = fileList.iterator();

                    while (fe.hasNext()) {
                        Element fileEntry = (Element) fe.next();
                        String filename = fileEntry.getText();

                        fileNames.add(filename);

                        pf.addFile(filename);
                    }
                }

                Element params = filter.getChild("params");

                if (params != null) {
                    List paramList = params.getChildren("param");
                    Iterator pe = paramList.iterator();
   
                    while (pe.hasNext()) {    
                        Element paramEntry = (Element) pe.next();
         
                        String param = paramEntry.getAttributeValue("name");
                        String value = paramEntry.getAttributeValue("value");
      
                        pf.addParam(param, value);
                    }
                }
        
                pf.setStatus(status);
                if (status.equals("active"))
                    pf.setActive(true);
                else
                    pf.setActive(false);
        
                pf.setName(name);
                pf.setDescription(description);
        
                if (removable.equals("true"))
                    pf.setRemovable(true);
                else
                    pf.setRemovable(false);
        
                pf.setId(id);
        
                filters.add(pf);      
            }
      
            // Process files
            for (Enumeration e = fileNames.elements(); e.hasMoreElements();) {
                String fileName = (String) e.nextElement();

                if (uniqFileNames.get(fileName) != null)
                    continue;
                else
                    uniqFileNames.put(fileName, "");
            
                fileName = fileName.substring(fileName.indexOf("/") + 1);

                String cont = getFileFromServer(fileName);
            
                // File is not there
                if (cont == null)
                    cont = "";
            
                GuiFile file = new GuiFile();

                file.setName(fileName);
                file.setContent(cont);
            
                // Add file Object to files Vector
                files.add(file);
            }
	   /*
        } catch (Exception ex) {
            //System.err.println(ex.getMessage());
            System.out.println("Exception: " + ex.getMessage());
            ex.printStackTrace(System.out);
        }
	*/
    
    }
  
    private void buildServerConfig(String xml) {

        String proxyHost = null;
        String proxyPort = null;
    
        try {
            // No validation
            SAXBuilder builder = new SAXBuilder(false);
            Document doc = builder.build(new StringReader(xml));

            // Get root element
            Element root = doc.getRootElement();

            String serverPort = root.getChild("port").getText();
  
            String log = root.getChild("log").getText();

            if (root.getChild("proxyhost") != null)
                proxyHost = root.getChild("proxyhost").getText();

            if (root.getChild("proxyport") != null)
                proxyPort = root.getChild("proxyport").getText();

            // Read admin settings
            boolean adminActive = false;
            String adminPort = null;
            String adminUser = null;
            String adminPass = null;
       
            Element admin = root.getChild("admin");

            if (admin != null) {
                adminActive = true;
                adminPort = admin.getChild("port").getText();
                adminUser = admin.getChild("user").getText();
                adminPass = admin.getChild("pass").getText();
          
            }
    
            // Build the Server Object
            serverObj = new GuiServerConfig(host);
            serverObj.setPort(serverPort);
            serverObj.setLog(log);
            serverObj.setProxyHost(proxyHost);
            serverObj.setProxyPort(proxyPort);
            serverObj.setConfigFile(xml);
       
            if (proxyHost != null && proxyPort != null)
                serverObj.setUseProxy(true);
            else
                serverObj.setUseProxy(false);
       
            adminObj = new GuiAdminConfig();
       
            // Build the Admin Object
            if (adminActive) {
                adminObj.setActive(true);
                adminObj.setPort(adminPort);
                adminObj.setUser(adminUser);
                adminObj.setPass(adminPass);
            } else
                adminObj.setActive(false);

        } catch (JDOMException jdome) {
            System.out.println(jdome.getMessage());
        }
    }

    public boolean removeRegExpFilter(GuiRegExpFilter obj) {
         if(obj.getType().equals("replace")) {
           return replaceFilters.remove(obj);
         }
         else {
           return sizeFilters.remove(obj);
         }
    }
    
    public boolean removeFilter(GuiFilter obj) {
         return filters.remove(obj);
    }
    
     public boolean removeHandler(GuiHandler obj) {
         return handlers.remove(obj);
    }

    public boolean resyncHandlersAndFiltersWithTree(JTree tree) {
        TreeModel model = tree.getModel();
	Object root = model.getRoot();

	Vector tmpReplaceFilters = new Vector();
        Vector tmpSizeFilters = new Vector();
        Vector tmpHandlers = new Vector();
        Vector tmpFilters = new Vector();
	Vector tmpFiles = new Vector();
	Hashtable tmpUniqFileNames = new Hashtable();
	
	boolean ret = walkTreeAndAssignToVectors(model, root, tmpHandlers, tmpFilters,
	                           tmpReplaceFilters, tmpSizeFilters, tmpFiles, tmpUniqFileNames);

        if(ret) {
	    this.replaceFilters = tmpReplaceFilters;
	    this.sizeFilters = tmpSizeFilters;
	    this.handlers = tmpHandlers;
	    this.filters = tmpFilters;
	    this.files = tmpFiles;
	    this.uniqFileNames = tmpUniqFileNames;
	}
	
	return ret;

    }
	

    private boolean walkTreeAndAssignToVectors( TreeModel model, Object o,
                 Vector handlers, Vector filters, Vector replace, Vector size, 
		 Vector files, Hashtable uniqFiles ) {
		 
	boolean ret = true;

        try {
		for( int i=0;  i < model.getChildCount( o );  i++ )
		{
			DefaultMutableTreeNode child = (DefaultMutableTreeNode) model.getChild(o, i);

			Object userObj = child.getUserObject();
			if(userObj instanceof GuiHandler) {
				handlers.add(userObj);
			}
			else if(userObj instanceof GuiFilter) {
				filters.add(userObj);
			}
			else if(userObj instanceof GuiRegExpFilter) {
   				GuiRegExpFilter reg = (GuiRegExpFilter) userObj;
				if(reg.getType().equals("replace")) {
				        replace.add(userObj);
				}
				else {
					size.add(userObj);
				 }
			}
			else if(userObj instanceof GuiFile) {
				GuiFile file = (GuiFile) userObj;
				if(uniqFiles.get(file.getName()) == null) {
    					uniqFiles.put(file.getName(), "");
					files.add(userObj);
				}
			}

   			if(!model.isLeaf(child)) {
				ret = walkTreeAndAssignToVectors(model, child,
			        	    handlers, filters, replace, size, files, uniqFiles );
			}
			
		}
	} catch(Exception ex) {
	     ret = false;
	}

	return ret;
     }

}
