/*
 * GuiBuilder.java
 *
 * Created on 19. Juli 2001, 20:15
 */

package org.paw.gui;


import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;


/**
 *
 * @author  Standard
 * @version 
 */
public class GuiBuilder {
    private MainWindow parent;
    private JToolBar toolBar;
    private JPanel contentPane;
    private JTree tree;
    private JComboBox comboBox;
    private Hashtable menuHash;
    private JSplitPane splitPane;

    private DefaultMutableTreeNode deleteFiltersNode;
    private DefaultMutableTreeNode sizeFiltersNode;
    private DefaultMutableTreeNode filesNode;
    private DefaultMutableTreeNode handlersNode;
    private DefaultMutableTreeNode filtersNode;
    
    /** Creates new GuiBuilder */
    public GuiBuilder(MainWindow parent) {
        this.parent = parent;
    }
 
    public void addWindowExitListener(Frame frame) {
        frame.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    parent.saveConfig();
                    System.exit(0);
                }
            }
        );
    }

    public void addHandler(GuiHandler handler) {
         handlersNode.add(new DefaultMutableTreeNode(handler));
         DefaultTreeModel model = (DefaultTreeModel) tree.getModel();
         model.reload(handlersNode);
    }
    
    public void addFilter(GuiFilter filter) {
         filtersNode.add(new DefaultMutableTreeNode(filter));
         DefaultTreeModel model = (DefaultTreeModel) tree.getModel();
         model.reload(filtersNode);
    }
    
    public void addReplaceFilter(GuiRegExpFilter filter) {
         deleteFiltersNode.add(new DefaultMutableTreeNode(filter));
         DefaultTreeModel model = (DefaultTreeModel) tree.getModel();
         model.reload(deleteFiltersNode);
    }

    public void addSizeFilter(GuiRegExpFilter filter) {
         sizeFiltersNode.add(new DefaultMutableTreeNode(filter));
         DefaultTreeModel model = (DefaultTreeModel) tree.getModel();
         model.reload(sizeFiltersNode);
    }
    
    public void addFile(GuiFile file) {
         filesNode.add(new DefaultMutableTreeNode(file));
         DefaultTreeModel model = (DefaultTreeModel) tree.getModel();
         model.reload(filesNode);
    }

    public void buildMenu() {
        JMenuBar menuBar = new JMenuBar();

        parent.getRootPane().setJMenuBar(menuBar);

        // File menu Start
        JMenu fileMenu = new JMenu("File");

        fileMenu.setMnemonic(KeyEvent.VK_F);

        menuBar.add(fileMenu);
       
        // Save item
        JMenuItem saveItem = new JMenuItem("Sync",
                new ImageIcon("gui_images/sync_small.gif"));

        saveItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
                ActionEvent.CTRL_MASK));

        saveItem.setActionCommand("Sync Configuration");
        saveItem.addActionListener(parent);
        fileMenu.add(saveItem);
       
        fileMenu.addSeparator();
       
        // Exit Item
        JMenuItem exitItem = new JMenuItem("Exit",
                new ImageIcon("gui_images/exit.gif"));

        exitItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,
                ActionEvent.CTRL_MASK));
        exitItem.setActionCommand("Exit");
        exitItem.addActionListener(parent);
        fileMenu.add(exitItem);
       
        // File menu end
       
        // Server menu start
        JMenu serverMenu = new JMenu("Server");

        serverMenu.setMnemonic(KeyEvent.VK_S);

        menuBar.add(serverMenu);
       
        // Connect Item
        JMenuItem connectItem = new JMenuItem("Connect",
                new ImageIcon("gui_images/connect_small.gif"));

        connectItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C,
                ActionEvent.CTRL_MASK));
        connectItem.setActionCommand("Connect");
        connectItem.addActionListener(parent);
        serverMenu.add(connectItem);
       
        serverMenu.addSeparator();

        // Show Serverlog Item
        JMenuItem serverLogItem = new JMenuItem("Show Serverlog",
                new ImageIcon("gui_images/showlog_small.gif"));

        serverLogItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L,
                ActionEvent.CTRL_MASK));
        serverLogItem.setActionCommand("Show Serverlog");
        serverLogItem.addActionListener(parent);
        serverMenu.add(serverLogItem);
	
	// Show Clearlog Item
        JMenuItem clearLogItem = new JMenuItem("Clear Serverlog",
                new ImageIcon("gui_images/clearlog_small.jpg"));

        clearLogItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R,
                ActionEvent.CTRL_MASK));
        clearLogItem.setActionCommand("Clear Serverlog");
        clearLogItem.addActionListener(parent);
        serverMenu.add(clearLogItem);
       
        serverMenu.addSeparator();
       
        // Status Item
        JMenuItem statusItem = new JMenuItem("Status",
                new ImageIcon("gui_images/inform_small.gif"));

        statusItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I,
                ActionEvent.CTRL_MASK));
        statusItem.setActionCommand("Server Status");
        statusItem.addActionListener(parent);
        serverMenu.add(statusItem);
       
        // Stop Item
        JMenuItem stopItem = new JMenuItem("Stop",
                new ImageIcon("gui_images/stop_small.gif"));

        stopItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D,
                ActionEvent.CTRL_MASK));
        stopItem.setActionCommand("Stop Server");
        stopItem.addActionListener(parent);
        serverMenu.add(stopItem);
       
        // Start Item
        JMenuItem startItem = new JMenuItem("Start",
                new ImageIcon("gui_images/start_small.gif"));

        startItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U,
                ActionEvent.CTRL_MASK));
        startItem.setActionCommand("Start Server");
        startItem.addActionListener(parent);
        serverMenu.add(startItem);
       
        // Shutdown Item
        JMenuItem shutdownItem = new JMenuItem("Shutdown",
                new ImageIcon("gui_images/shutdown_small.gif"));

        shutdownItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,
                ActionEvent.CTRL_MASK));
        shutdownItem.setActionCommand("Shutdown Server");
        shutdownItem.addActionListener(parent);
        serverMenu.add(shutdownItem);
  
        // Server menu end

        // Import menu start
        JMenu importMenu = new JMenu("Import");
        importMenu.setMnemonic(KeyEvent.VK_I);
        menuBar.add(importMenu);

        JMenuItem importItem = new JMenuItem("Handlers and Filters",
            new ImageIcon("gui_images/import.gif"));
        importItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,
                ActionEvent.CTRL_MASK));

        importItem.setActionCommand("ImportHandlersAndFilters");
        importItem.addActionListener(parent);
        importMenu.add(importItem);
        // Import menu start ende
	
	// Insert menu start
        JMenu insertMenu = new JMenu("Insert");
        insertMenu.setMnemonic(KeyEvent.VK_N);
        menuBar.add(insertMenu);

        JMenu insertNewItem = new JMenu("New");
	insertNewItem.setIcon(new ImageIcon("gui_images/new_handler_filter.gif"));
	
	JMenuItem newHandler = new JMenuItem("Handler",
	                      new ImageIcon("gui_images/document.gif"));
        newHandler.setActionCommand("NewHandler");
        newHandler.addActionListener(parent);
	insertNewItem.add(newHandler);
	
	JMenuItem newFilter = new JMenuItem("Filter",
	                  new ImageIcon("gui_images/document.gif"));
        newFilter.setActionCommand("NewFilter");
        newFilter.addActionListener(parent);
	insertNewItem.add(newFilter);
	
	JMenuItem newReplaceFilter = new JMenuItem("RexExp Replace Filter",
	                  new ImageIcon("gui_images/document.gif"));
        newReplaceFilter.setActionCommand("NewReplaceFilter");
        newReplaceFilter.addActionListener(parent);
	insertNewItem.add(newReplaceFilter);

        JMenuItem newSizeFilter = new JMenuItem("RexExp Size Filter",
	                 new ImageIcon("gui_images/document.gif"));
        newSizeFilter.setActionCommand("NewSizeFilter");
        newSizeFilter.addActionListener(parent);
	insertNewItem.add(newSizeFilter);

        insertMenu.add(insertNewItem);
        // Insert menu start ende
       
        // Help menu start
        JMenu helpMenu = new JMenu("Help");

        helpMenu.setMnemonic(KeyEvent.VK_H);

        menuBar.add(helpMenu);
        // Help Item
        JMenuItem helpItem = new JMenuItem("Help", 
                new ImageIcon("gui_images/book-open.gif"));

        helpItem.setActionCommand("Help");
        helpItem.addActionListener(parent);
        helpMenu.add(helpItem);
       
        helpMenu.addSeparator();
       
        // About Item
        JMenuItem aboutItem = new JMenuItem("About", 
                new ImageIcon("gui_images/about.gif"));

        aboutItem.setActionCommand("About");
        aboutItem.addActionListener(parent);
        helpMenu.add(aboutItem);
       
        // Help menu end
    }
    
    public void setFrameIcon(String icon) {
        Image frameImage = Toolkit.getDefaultToolkit().getImage(icon);

        parent.setIconImage(frameImage);
    }
    
    public JToolBar createToolBar() {
        contentPane = (JPanel) parent.getContentPane();

        toolBar = new JToolBar();
        toolBar.setFloatable(true);
        
        contentPane.add("North", toolBar);
        
        return toolBar;
    }
    
    public JButton addToToolBar(String name, String text, String image) {
        JButton newButton = new JButton(text, new ImageIcon(image));

        newButton.setToolTipText(name);
        newButton.addActionListener(parent);
        newButton.setActionCommand(name);
        newButton.setMargin(new Insets(0, 0, 0, 0));
        newButton.setBorderPainted(false);
        newButton.addMouseListener(new FlatLookMouseListener());
        
        toolBar.add(newButton);
        
        return newButton;
    }
    
    public JComboBox createComboBox() {
        comboBox = new JComboBox();
        toolBar.add(comboBox);
        
        return comboBox;
    }
       
    public void setToolBarIconAndTitle(final String icon, final String title) {
        toolBar.setUI(new javax.swing.plaf.metal.MetalToolBarUI() {
                protected JFrame createFloatingFrame(JToolBar tb) {
                    JFrame f = super.createFloatingFrame(tb);

                    f.setResizable(false);
                    f.setIconImage(Toolkit.getDefaultToolkit().getImage(icon));
                    f.setTitle(title);
                    return f;
                }
            }
        );
    }

    public JTree getTree() {
        return tree;
    }
    
    public void createSplitPane() {     
        //JTree tree = createTree();
        tree = new JTree();
        tree.setEditable(true);
        
        CustomTreeCellRenderer renderer = new CustomTreeCellRenderer(); 

        tree.setCellRenderer(renderer);
      
        DefaultMutableTreeNode root = new DefaultMutableTreeNode("Not connected");
        DefaultTreeModel model = (DefaultTreeModel) tree.getModel(); 

        model.setRoot(root);
        
        JPanel panel = new JPanel();

        panel.setLayout(new BorderLayout());
        
        JScrollPane sp1 = new JScrollPane(tree); 
        
        JLabel title = new JLabel("  PAW Server");

        title.setForeground(Color.white);
        title.setFont(new Font("SansSerif", Font.BOLD, Icons.BigIconHeight));
        
        JToolBar tBar = new JToolBar();

        tBar.setFloatable(false);
        tBar.setBackground(Color.gray);
        
        tBar.add(title);
        panel.add(tBar, "North");
        
        panel.add(sp1, "Center");

        //TextArea textArea = new JTextArea();

        //JScrollPane sp2 = new JScrollPane(textArea);

     
        splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, 
                    panel, new EmptyPanel());
    
        splitPane.setOneTouchExpandable(true);
        splitPane.setDividerLocation(200);

        contentPane.add(splitPane, "Center");
        
        contentPane.add(new JLabel(), "South");
        
    }
    
    public void updateSplitPane(JComponent panel) {
        int divLocation = splitPane.getDividerLocation();

        splitPane.setRightComponent(panel);
        splitPane.setDividerLocation(divLocation);
    }
    
    public void expandTree() {
        for (int row = 0; row < tree.getRowCount(); row++)
            tree.expandRow(row);
    }
    
    public void collapseTree() {
        int collapseLevel = 1;
       
        for (int row = 0; row < tree.getRowCount(); row++) {
            if (row > collapseLevel)
                tree.collapseRow(row);
        }
    }

    public void clearTreeSelection() {
        tree.clearSelection();
    }


    private void setupTreeMenus() {
        menuHash = new Hashtable();

        // Standard node menu
        String[] label_node_enable  =  {"enable:", "up", "down"};
        String[] label_node_disable  =  {"disable:", "up", "down"};


        String[] action_node = {"SwitchNode", "MoveNodeUp", "MoveNodeDown"};

	String[] images_node_enable  =  {"gui_images/circle_green.gif", "gui_images/up.gif", "gui_images/down.gif"};
        String[] images_node_disable  =  {"gui_images/circle_red.gif", "gui_images/up.gif", "gui_images/down.gif"};

        JPopupMenu enablePopup = createPopupMenu(label_node_enable, action_node,
	                               images_node_enable);
        menuHash.put("enable", enablePopup);

        JPopupMenu disablePopup = createPopupMenu(label_node_disable, action_node, 
	                               images_node_disable);
        menuHash.put("disable", disablePopup);

      
        /* ReqExpFilter popups */
        String[] label_enable_delete = {"enable:", "delete:", "edit:" ,"export:", "up", "down"};
        String[] label_disable_delete = {"disable:", "delete:", "edit:", "export:", "up", "down"};

	String[] images_enable_delete = {"gui_images/circle_green.gif", "gui_images/delete_filter.gif", 
	                                 "gui_images/edit.gif", "gui_images/export.gif",
					 "gui_images/up.gif", "gui_images/down.gif"};
        String[] images_disable_delete = {"gui_images/circle_red.gif", "gui_images/delete_filter.gif",
	                                  "gui_images/edit.gif", "gui_images/export.gif",
					  "gui_images/up.gif", "gui_images/down.gif"};

        String[] action_switchnode= {"SwitchNode", "DeleteNode", "EditFilterHandler", "Export",
	                             "MoveNodeUp", "MoveNodeDown"};
        JPopupMenu enableDeletePopup = createPopupMenu(label_enable_delete, action_switchnode,
	                                               images_enable_delete);
        menuHash.put("delete_enable", enableDeletePopup);
        /* ------------- */
        JPopupMenu deleteDisablePopup = createPopupMenu(label_disable_delete, action_switchnode,
	                                                images_disable_delete);
        menuHash.put("delete_disable", deleteDisablePopup);
        /* ReqExpFilter popups END*/

        // Expand/Collapse menu
        
        String[] label_exp_coll  =  {"expand", "collapse"};
        String[] action_exp_coll = {"Expand tree", "Collapse tree"};
	String[] images_exp_coll = {"gui_images/plus.gif", "gui_images/minus.gif"};
        JPopupMenu serverPopup = createPopupMenu(label_exp_coll, action_exp_coll,
	                                         images_exp_coll);
        menuHash.put("serverPopup", serverPopup);
      
    }
 
    public void setupTree(GuiServerConfig server, GuiAdminConfig admin,
        Vector handlers, Vector filters,
        Vector replaceFilters, Vector sizeFilters,
        Vector files) {

        /*
         Before setting up the tree we delete the Panel in the right half of
         the splitpane
         */
        updateSplitPane(new EmptyPanel());
       
        DefaultTreeModel model = (DefaultTreeModel) tree.getModel(); 
      
        tree.addTreeSelectionListener(parent);

        setupTreeMenus();

        tree.addMouseListener(new TreeMouseListener(menuHash));
       
        // Ad server node as root 
        DefaultMutableTreeNode root = new DefaultMutableTreeNode(server);

        DefaultMutableTreeNode adminNode = new DefaultMutableTreeNode(admin);
 
        root.add(adminNode);

        // Insert Handlers
        handlersNode = new DefaultMutableTreeNode("Handlers");

        root.add(handlersNode);
      
        for (Enumeration e = handlers.elements(); e.hasMoreElements();) {
            Object obj = e.nextElement();

            handlersNode.add(new DefaultMutableTreeNode(obj));
        }
      
        // Insert Filters
        filtersNode = new DefaultMutableTreeNode("Filters");

        root.add(filtersNode);
      
        // This is the hook for the RegExp filters
        DefaultMutableTreeNode regFiltersNode = null;
     
        for (Enumeration e = filters.elements(); e.hasMoreElements();) {
            Object obj = e.nextElement();
            GuiFilter filter = (GuiFilter) obj;
            DefaultMutableTreeNode node = new DefaultMutableTreeNode(obj);
         
            if (filter.hasClass("org.paw.filter.PawRegFilter"))
                regFiltersNode = node;
         
            filtersNode.add(node);
        }
   
        AdminConnection currentConnection = parent.getCurrentConnection();
      
        if (currentConnection.hasFilterClass("org.paw.filter.PawRegFilter") &&
            regFiltersNode != null) {
            // Insert Delete Filters
            deleteFiltersNode = new DefaultMutableTreeNode("Replace Filters");

            regFiltersNode.add(deleteFiltersNode);

            for (Enumeration e = replaceFilters.elements(); e.hasMoreElements();) {
                Object obj = e.nextElement();

                deleteFiltersNode.add(new DefaultMutableTreeNode(obj));
            }

            // Insert Size Filters
           sizeFiltersNode =
                new DefaultMutableTreeNode("Size Filters");

            regFiltersNode.add(sizeFiltersNode);

            for (Enumeration e = sizeFilters.elements(); e.hasMoreElements();) {
                Object obj = e.nextElement();

                sizeFiltersNode.add(new DefaultMutableTreeNode(obj));
            }
        }

        // Insert Files
        filesNode = new DefaultMutableTreeNode("Files");

        root.add(filesNode);

        for (Enumeration e = files.elements(); e.hasMoreElements();) {
            Object obj = e.nextElement();

            filesNode.add(new DefaultMutableTreeNode(obj));
        }

        model.setRoot(root);

        // This is where the TreeRenderer can also be placed
        DefaultTreeCellEditor cellEditor = new DefaultTreeCellEditor(tree, new DefaultTreeCellRenderer(),
                new LeafEditor(tree, new JTextField()));

        tree.setCellEditor(cellEditor);

    }


    private JPopupMenu createPopupMenu(String[] label, String[] action, String[] images)
    {
        JPopupMenu popup = new JPopupMenu ();
        JMenuItem item;

        for(int i = 0; i < label.length; i++) {
	    boolean separator = false;
	    if(label[i].endsWith(":")) {
               separator = true;
	       label[i] = label[i].substring(0, label[i].indexOf(":"));
	    }
	    
	    if(images[i] == null) {
               item = new JMenuItem(label[i]);
	    }
	    else {
	       item = new JMenuItem(label[i], new ImageIcon(images[i]));
	    }
            item.addActionListener(parent);
            item.setActionCommand(action[i]);
            popup.add(item);
            if(separator) {
                popup.addSeparator();
            }
        }

        return popup;
    }

}


class LeafEditor extends DefaultCellEditor {
    private JTree tree;

    public LeafEditor(JTree tree, JTextField textField) {
        super(textField);
        this.tree = tree;
    }
 
    public boolean isCellEditable(EventObject e) {
        boolean ret = false;

        if (e instanceof MouseEvent) {
            MouseEvent me = (MouseEvent) e;
    
            if (me.getClickCount() == 3) {      
                TreePath path = tree.getPathForLocation(me.getX(), me.getY());

                DefaultMutableTreeNode node = (DefaultMutableTreeNode) path.getLastPathComponent();

                /*    
                 if((node.getUserObject() instanceof GuiFilter) ||
                 (node.getUserObject() instanceof GuiFile))
                 ret = true;
                 else
                 ret = false;
                 */ 
            
                //ret = node.isLeaf();
            
                // In this early version we do not allow editing !!!!
            }
        }

        return ret;
    }
}


class CustomTreeCellRenderer extends DefaultTreeCellRenderer {
    public CustomTreeCellRenderer() {
        // Set default icons
        setOpenIcon(new ImageIcon("gui_images/folder_open.gif"));
        setClosedIcon(new ImageIcon("gui_images/folder.gif"));
        //setLeafIcon(new ImageIcon("gui_images/l.gif"));

    }

    public Component getTreeCellRendererComponent(
        JTree tree, Object value, boolean sel,
        boolean expanded, boolean leaf, int row,
        boolean hasFocus) {

        super.getTreeCellRendererComponent(tree, value, sel, expanded,
            leaf, row, hasFocus);
        DefaultMutableTreeNode node =
            (DefaultMutableTreeNode) value;

        Object selectedItem = node.getUserObject();

        //if (selectedItem instanceof Join)

            

        if (node.isRoot())
            setIcon(new ImageIcon("gui_images/paw.gif"));

        if (selectedItem instanceof GuiAdminConfig) {
            setText("Admin Server");
            setIcon(new ImageIcon("gui_images/executeproject_small.gif"));
        }
            
        if (selectedItem instanceof GuiServerConfig) {
            GuiServerConfig sc = (GuiServerConfig) selectedItem;

            setText(sc.getServerName());
        }

        if (selectedItem instanceof GuiRegExpFilter) {
            GuiRegExpFilter filter = (GuiRegExpFilter) selectedItem;

            setText(filter.getName());
              
            if (filter.isActive())
                setIcon(new ImageIcon("gui_images/filteractive.gif"));
            else
                setIcon(new ImageIcon("gui_images/filterinactive.gif"));
       
        }
            
        if (selectedItem instanceof GuiHandler) {
            GuiHandler handler = (GuiHandler) selectedItem;

            setText(handler.getName());
              
            if (handler.isActive())
                setIcon(new ImageIcon("gui_images/enable_handler.gif"));
            else
                setIcon(new ImageIcon("gui_images/disable_handler.gif"));
        }
            
        if (selectedItem instanceof GuiFilter) {
            GuiFilter filter = (GuiFilter) selectedItem;

            setText(filter.getName());
              
            if (filter.isActive())
                setIcon(new ImageIcon("gui_images/enable_handler.gif"));
            else
                setIcon(new ImageIcon("gui_images/disable_handler.gif"));
        }

        if (selectedItem instanceof GuiFile) {
            GuiFile file = (GuiFile) selectedItem;

            setText(file.getName());
              
            setIcon(new ImageIcon("gui_images/document.gif"));
        }

        if (selectedItem instanceof String) {
            String s = (String) selectedItem;

            /*
             if(s.equals("Replace Filters"))
             setIcon(new ImageIcon("gui_images/magnify.gif"));
             
             if(s.equals("Size Filters"))
             setIcon(new ImageIcon("gui_images/magnify.gif"));
             **/

            /* Will be handled by default icons
             if(s.equals("Files"))
             setIcon(new ImageIcon("gui_images/folder.gif"));
             
             if(s.equals("Handlers"))
             setIcon(new ImageIcon("gui_images/folder.gif"));
             
             if(s.equals("Filters"))
             setIcon(new ImageIcon("gui_images/folder.gif"));
             */
        }
    
        return this;

    }

}


final class TreeMouseListener extends MouseAdapter {

    Hashtable menus;

    public TreeMouseListener(Hashtable menus) {
        //this.super();
        this.menus = menus;
    }
  
    public void mousePressed(MouseEvent e) {
        JTree tree = (JTree) e.getSource();

        TreePath path = tree.getPathForLocation(e.getX(), e.getY());
        
        if(path == null) {
          return;
        }

        tree.setSelectionPath(path);

        mouseReleased(e);
    }
        
    public void mouseReleased(MouseEvent e) {
        JTree tree = (JTree) e.getSource();
        Object o = tree.getLastSelectedPathComponent();

        String menuStringPrefix = "";

        // If nothing has been selected
        if (o == null)
            return;

        DefaultMutableTreeNode node = (DefaultMutableTreeNode) o;

        Object object = node.getUserObject();

        JPopupMenu popup;  

        boolean isHandlerOrFilter = object instanceof GuiHandler ||
            object instanceof GuiFilter ||
            object instanceof GuiRegExpFilter;

        boolean trigger = e.isPopupTrigger();
            
        //JComponent tree = (JComponent)e.getSource ();
            
        if (isHandlerOrFilter && trigger) {
            // state: true = active false=inactive
            boolean state = true;
                
            if (object instanceof GuiHandler) {
                GuiHandler handler = (GuiHandler) object;
                   
                state = handler.isActive() ? false : true;
                
                   if(handler.isRemovable()) {
                      menuStringPrefix = "delete_";
                    }
            }
                
            if (object instanceof GuiFilter) {
                GuiFilter filter = (GuiFilter) object;
                   
                state = filter.isActive() ? false : true;
    
                if(filter.isRemovable()) {
                      menuStringPrefix = "delete_";
                }
            }
                
            if (object instanceof GuiRegExpFilter) {
                GuiRegExpFilter regFilter = (GuiRegExpFilter) object;
                   
                state = regFilter.isActive() ? false : true;
                menuStringPrefix = "delete_";
            }

            popup = (JPopupMenu) menus.get(state ? menuStringPrefix + "enable" : menuStringPrefix + "disable");
                
            //popup.show (tree, e.getX () - tree.getVisibleRect().x,
            //     e.getY () - tree.getVisibleRect().y);
            popup.show(tree, e.getX(),
                e.getY());

        }
            
        if ((object instanceof GuiServerConfig) && trigger) {
            popup = (JPopupMenu) menus.get("serverPopup");
            popup.show(tree, e.getX(),
                e.getY());
        }
    }
    

}

