/*
 * HandlerInfoPanel.java
 *
 * Created on 29. Juli 2001, 08:52
 */

package org.paw.gui;


import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.text.html.*;


/**
 *
 * @author  Standard
 * @version 
 */
public class HandlerInfoPanel extends JPanel 
    implements HyperlinkListener{
    private GuiHandler handler;
    private JPanel panel;
    private GridBagLayout gbl;
    private GridBagConstraints gbc;

    private static Image bgImage;
    private static int backgroundImageWidth;
    private static int backgroundImageHeight;
    private String bgImageName = "backgrounds/infoBackground.jpg";
    private Color background = Color.white;
    
    /** Creates newEmptyPanel */
    public HandlerInfoPanel(DefaultMutableTreeNode node) {
        super();
        
        Object object = node.getUserObject();

        handler = (GuiHandler) object;
        
        setLayout(new BorderLayout());

        // Initialize background image
        bgImage = Toolkit.getDefaultToolkit().getImage(bgImageName);
        MediaTracker tracker = new MediaTracker(this);

        tracker.addImage(bgImage, 0); 
        try {
            tracker.waitForAll();
        } catch (InterruptedException e) {}
   
        backgroundImageWidth = bgImage.getWidth(null);
        backgroundImageHeight = bgImage.getHeight(null);
       
        setupPanel();
    }

    private void setupPanel() {
        JToolBar toolBar = new JToolBar();      

        toolBar.setFloatable(false);
        toolBar.setBackground(Color.gray);
        
        JLabel title = new JLabel("  " + handler.getName());

        title.setForeground(Color.white);
        title.setFont(new Font("SansSerif", Font.BOLD, Icons.BigIconHeight));
        
        toolBar.add(title);
        
        add(toolBar, "North");

        panel = new JPanel() {
                    public void paintComponent(Graphics g) {
                        super.paintComponent(g); 
                        g.drawImage(bgImage, 0, 0, backgroundImageWidth, backgroundImageHeight, null);
                    }
                };

        panel.setBackground(background);
        
        JScrollPane scroller = new JScrollPane(panel);
        
        gbl = new GridBagLayout();
        gbc = new GridBagConstraints();
        panel.setLayout(gbl);
        
        add(scroller, "Center");
        
        addFilterInfo();
    }
        
    private void addFilterInfo() {
        if(handler.getDescription().indexOf("@URL") == -1) {
            JTextArea descriptionContent = new JTextArea(handler.getDescription());

            descriptionContent.setBorder(null);
            descriptionContent.setEditable(false);
            //descriptionContent.setBackground(UIManager.getColor("Label.background"));
            descriptionContent.setBackground(Color.white);
            descriptionContent.setForeground(UIManager.getColor("Label.foreground"));
            descriptionContent.setFont(UIManager.getFont("Label.font"));

            addComponent(descriptionContent, GridBagConstraints.REMAINDER);
        }
        else {
	   int index = handler.getDescription().indexOf("@URL");
	   index = handler.getDescription().indexOf("\"", index);
	   String url =handler.getDescription().substring(index + 1,
	                         handler.getDescription().indexOf("\"", index + 1));
           JEditorPane descriptionContent = new JEditorPane();
	   descriptionContent.addHyperlinkListener(this);

	   if(!url.startsWith("http://")) {
	      url = "file:///" + System.getProperty("user.dir") + "/" + url;
	   }

	   try {
	     descriptionContent.setPage(url);
	   } catch(Exception e) { e.printStackTrace(); }

           descriptionContent.setEditable(false);

	   addComponent(descriptionContent, GridBagConstraints.REMAINDER);

	 }

        addComponent(new JLabel("Files used:"), GridBagConstraints.REMAINDER,
            20, 1, 5, 1);
        
        for (Enumeration e = handler.files.keys(); e.hasMoreElements();) {
            String file = (String) e.nextElement();

            if (file.startsWith("conf"))
                file = file.substring(5);
          
            addComponent(new JLabel(file,
                    new ImageIcon("gui_images/document.gif"),
                    SwingConstants.LEFT),
                GridBagConstraints.REMAINDER);
        }
       
    }
    
    private void addComponent(Component component, int width) {
        gbc.fill = GridBagConstraints.NONE;
        gbc.gridwidth = width;
        gbc.insets = new Insets(1, 1, 1, 1);
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbl.setConstraints(component, gbc);
        panel.add(component);
    }
    
    private void addComponent(Component component, int width,
        int top, int left, int bottom, int right) {
        gbc.fill = GridBagConstraints.NONE;
        gbc.gridwidth = width;
        gbc.insets = new Insets(top, left, bottom, right);
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbl.setConstraints(component, gbc);
        panel.add(component);
    }      

    public void hyperlinkUpdate(HyperlinkEvent e) {
        if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
            JEditorPane pane = (JEditorPane) e.getSource();

            if (e instanceof HTMLFrameHyperlinkEvent) {
                HTMLFrameHyperlinkEvent  evt = (HTMLFrameHyperlinkEvent) e;
                HTMLDocument doc = (HTMLDocument) pane.getDocument();

                doc.processHTMLFrameHyperlinkEvent(evt);
            } else {
                try {
                    pane.setPage(e.getURL());
                } catch (Throwable t) {
                    t.printStackTrace();
                }
            }
        }
    }

}
