/*
 * TextEdit.java
 *
 * Created on 28. Juli 2001, 18:35
 */

package org.paw.gui;


import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.text.*;


/**
 *
 * @author  Standard
 * @version 
 */
public class TextEdit extends JPanel implements DocumentListener, ActionListener {
    private DefaultMutableTreeNode node = null;
    private String content = null;
    JTextPane textArea;
    JLabel title;
    MainWindow parent;
    ColourThread colourThread;
    private String type = null;
 
    /** Creates new TextEdit */
    public TextEdit(String content, DefaultMutableTreeNode node, Object parent) {
        super();
        this.content = content;
        this.node = node;
	this.parent = (MainWindow) parent;

	GuiFile file = null;


	if (node.getUserObject() instanceof GuiFile) {
            Object object = node.getUserObject();
            file = (GuiFile) object;
	}
        
        setLayout(new BorderLayout());
        
        JToolBar toolBar = new JToolBar();      

        toolBar.setFloatable(false);
        toolBar.setBackground(Color.gray);
        
        JButton saveButton = new JButton(new ImageIcon("gui_images/apply.gif"));

        saveButton.setBackground(Color.gray);
        saveButton.setToolTipText("Apply");
        saveButton.addActionListener(this);
        saveButton.setActionCommand("Apply");
        saveButton.setBorderPainted(false);
        saveButton.addMouseListener(new FlatLookMouseListener());

        toolBar.add(saveButton);

	title = new JLabel();

	if (node.getUserObject() instanceof GuiFile) {
            title.setText(file.getName());
	    type="text";
	}
	else if(node.getUserObject() instanceof GuiRegExpFilter) {
	    GuiRegExpFilter filter = (GuiRegExpFilter) node.getUserObject();
            title.setText(filter.getName());
	    type="xml";
	}
	else if(node.getUserObject() instanceof GuiFilter) {
	    GuiFilter filter = (GuiFilter) node.getUserObject();
            title.setText(filter.getName());
            type="xml";
	}
	else if(node.getUserObject() instanceof GuiHandler) {
	    GuiHandler handler = (GuiHandler) node.getUserObject();
            title.setText(handler.getName());
            type="xml";
	}
	else {
		// If type is unknown
		return;
	}

        title.setForeground(Color.white);
        title.setFont(new Font("SansSerif", Font.BOLD, Icons.BigIconHeight));
        
        toolBar.addSeparator();
        toolBar.addSeparator();
        toolBar.addSeparator();
        toolBar.addSeparator();
        toolBar.add(title);
        
        add(toolBar, "North");
        
        setupEditor();

    }

    private void setupEditor() {
        textArea = new JTextPane();
	textArea.setText(content);
        textArea.setCaretPosition(0);
        textArea.getDocument().addDocumentListener(this);
 
	 colourThread = new ColourThread(textArea, type);
	 colourThread.start();
	 colourThread.hasChanged();
        
         JScrollPane scroller = new JScrollPane(textArea);

        add(scroller, "Center");   
             
        //JScrollBar hScrollBar = scroller.getHorizontalScrollBar();
        //hScrollBar.setValue(0);
        
        //JScrollBar vScrollBar = scroller.getVerticalScrollBar();
        //vScrollBar.setValue(0);
        
        //scroller.getHorizontalScrollBar().setValue(0);
        
   
    }
    
    public String getContent() {
        return textArea.getText();
    }
    
    public DefaultMutableTreeNode getNode() {
        return node;
    }
    
    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals("Apply")) {    
            Object object = node.getUserObject();
	    
	    DefaultTreeModel model = (DefaultTreeModel) parent.guiBuilder.getTree().getModel();

            if (object instanceof GuiFile) {
                GuiFile file = (GuiFile) object;

                file.setContent(textArea.getText());
            }
	    else if (object instanceof GuiHandler) {
       		Vector handlers = parent.currentConnection.getHandlers();

	       	int handlersBefore = parent.currentConnection.getHandlers().size();
		
		try {
         	   parent.currentConnection.buildHandlerConfig(textArea.getText());
		}
		catch(Exception eh) {
			JOptionPane.showMessageDialog(this,
			eh.getMessage(),
                    	"Error",
                    	JOptionPane.WARNING_MESSAGE);
		}

	       	if(handlersBefore < parent.currentConnection.getHandlers().size()) {
	        	int i = parent.currentConnection.getHandlers().size() - 1;
	         	GuiHandler handler = (GuiHandler) handlers.elementAt(i);
           		parent.guiBuilder.addHandler(handler);

		   	DefaultMutableTreeNode nodeParent = (DefaultMutableTreeNode) this.node.getParent();
	     		int index = nodeParent.getIndex(node);
	       		int numChildren = nodeParent.getChildCount();

          		DefaultMutableTreeNode newNode = (DefaultMutableTreeNode) nodeParent.getChildAt(numChildren - 1);

		   	nodeParent.insert(newNode, index);

	     		model.removeNodeFromParent(this.node);
        		parent.currentConnection.removeHandler((GuiHandler) object);

		   	model.reload(nodeParent);
		   	parent.guiBuilder.getTree().treeDidChange();

		   	parent.currentConnection.resyncHandlersAndFiltersWithTree(parent.guiBuilder.getTree());

			this.node = newNode;
			this.title.setText(handler.getName());

	     	}
	    }
	    else if (object instanceof GuiFilter) {
       		Vector filters = parent.currentConnection.getFilters();

	       	int filtersBefore = parent.currentConnection.getFilters().size();
		
		try {
         	   parent.currentConnection.buildFilterConfig(textArea.getText());
		}
		catch(Exception eh) {
			JOptionPane.showMessageDialog(this,
			eh.getMessage(),
                    	"Error",
                    	JOptionPane.WARNING_MESSAGE);
		}

	       	if(filtersBefore < parent.currentConnection.getFilters().size()) {
	        	int i = parent.currentConnection.getFilters().size() - 1;
	         	GuiFilter filter = (GuiFilter) filters.elementAt(i);
           		parent.guiBuilder.addFilter(filter);

		   	DefaultMutableTreeNode nodeParent = (DefaultMutableTreeNode) this.node.getParent();
	     		int index = nodeParent.getIndex(node);
	       		int numChildren = nodeParent.getChildCount();

          		DefaultMutableTreeNode newNode = (DefaultMutableTreeNode) nodeParent.getChildAt(numChildren - 1);

		   	nodeParent.insert(newNode, index);

	     		model.removeNodeFromParent(this.node);
        		parent.currentConnection.removeFilter((GuiFilter) object);

		   	model.reload(nodeParent);
		   	parent.guiBuilder.getTree().treeDidChange();

		   	parent.currentConnection.resyncHandlersAndFiltersWithTree(parent.guiBuilder.getTree());

			this.node = newNode;
			this.title.setText(filter.getName());

	     	}
	    }
	    else if (object instanceof GuiRegExpFilter) {
	    
               GuiRegExpFilter regFilter = (GuiRegExpFilter) object;
	       if(regFilter.getType().equals("replace")) {
	       		Vector replaceFilters = parent.currentConnection.getReplaceFilters();

	       		int regFiltersBefore = parent.currentConnection.getReplaceFilters().size();
			
			try {
               		   parent.currentConnection.buildRegFilterConfig(textArea.getText());
			}
			catch(Exception eh) {
				JOptionPane.showMessageDialog(this,
				eh.getMessage(),
                    		"Error",
                    		JOptionPane.WARNING_MESSAGE);
			}

	       		if(regFiltersBefore < parent.currentConnection.getReplaceFilters().size()) {
	           		int i = parent.currentConnection.getReplaceFilters().size() - 1;
	           		GuiRegExpFilter filter = (GuiRegExpFilter) replaceFilters.elementAt(i);
                   		parent.guiBuilder.addReplaceFilter(filter);

		   		DefaultMutableTreeNode nodeParent = (DefaultMutableTreeNode) this.node.getParent();
	           		int index = nodeParent.getIndex(node);
	           		int numChildren = nodeParent.getChildCount();

                   		DefaultMutableTreeNode newNode = (DefaultMutableTreeNode) nodeParent.getChildAt(numChildren - 1);

		   		nodeParent.insert(newNode, index);

	           		model.removeNodeFromParent(this.node);
                   		parent.currentConnection.removeRegExpFilter((GuiRegExpFilter) object);

		   		model.reload(nodeParent);
		   		parent.guiBuilder.getTree().treeDidChange();

		   		parent.currentConnection.resyncHandlersAndFiltersWithTree(parent.guiBuilder.getTree());
				
				this.node = newNode;
				this.title.setText(filter.getName());

	       		}
		}
		else {
			Vector sizeFilters = parent.currentConnection.getSizeFilters();

	       		int regFiltersBefore = parent.currentConnection.getSizeFilters().size();
			
			try {
               		   parent.currentConnection.buildRegFilterConfig(textArea.getText());
			}
			catch(Exception eh) {
				JOptionPane.showMessageDialog(this,
				eh.getMessage(),
                    		"Error",
                    		JOptionPane.WARNING_MESSAGE);
			}


	       		if(regFiltersBefore < parent.currentConnection.getSizeFilters().size()) {
	           		int i = parent.currentConnection.getSizeFilters().size() - 1;
	           		GuiRegExpFilter filter = (GuiRegExpFilter) sizeFilters.elementAt(i);
                   		parent.guiBuilder.addSizeFilter(filter);

		   		DefaultMutableTreeNode nodeParent = (DefaultMutableTreeNode) this.node.getParent();
	           		int index = nodeParent.getIndex(node);
	           		int numChildren = nodeParent.getChildCount();

                   		DefaultMutableTreeNode newNode = (DefaultMutableTreeNode) nodeParent.getChildAt(numChildren - 1);

		   		nodeParent.insert(newNode, index);

	           		model.removeNodeFromParent(this.node);
                   		parent.currentConnection.removeRegExpFilter((GuiRegExpFilter) object);

		   		model.reload(nodeParent);
		   		parent.guiBuilder.getTree().treeDidChange();

		   		parent.currentConnection.resyncHandlersAndFiltersWithTree(parent.guiBuilder.getTree());
				
				this.node = newNode;
				this.title.setText(filter.getName());

	       		}
		}
            }
        }
    }

    // Handle insertions into the text field
    public void	insertUpdate( DocumentEvent event )
    {
		colourThread.hasChanged();
    }

    // Handle deletions	from the text field
    public void removeUpdate( DocumentEvent event )
    {
        	colourThread.hasChanged();
    }

    // Handle changes to the text field
    public void changedUpdate( DocumentEvent event )
    {
		// Nothing to do here
    }

   

}

class ColourThread extends Thread {
       private JTextPane pane;
       private boolean recolour = false;
       private String type;

       public ColourThread(JTextPane pane, String type) {
	       this.pane = pane;
	       this.type = type;
       }
       
       public void run() {
	       for(;;) {
		       
		       if(recolour) {
			 recolour = false;
                         doSyntaxColouring();
		       }

		       try {
		         sleep(100);
		       } catch(Exception e) {  e.printStackTrace(); }
	       }
       }
	
       public void hasChanged() {
	       recolour = true;
       }


 private void doSyntaxColouring() {
         SimpleAttributeSet setNormal = new SimpleAttributeSet();
         setNormal.addAttribute(StyleConstants.CharacterConstants.Foreground,
			   Color.black);

         SimpleAttributeSet setString = new SimpleAttributeSet();
         setString.addAttribute(StyleConstants.CharacterConstants.Foreground,
			   Color.red);
	 
         SimpleAttributeSet setXml = new SimpleAttributeSet();
	 //setXml.addAttribute(StyleConstants.CharacterConstants.Bold,
	 //		   Boolean.TRUE);
         setXml.addAttribute(StyleConstants.CharacterConstants.Foreground,
			   Color.blue);

         SimpleAttributeSet setText = new SimpleAttributeSet();
         setText.addAttribute(StyleConstants.CharacterConstants.Foreground,
			   Color.blue);
	 
	 DefaultStyledDocument document = (DefaultStyledDocument) pane.getDocument();
	 document.putProperty(DefaultEditorKit.EndOfLineStringProperty, "\n");
	 String text = pane.getText();
	 
	 int found;
	 int index;

	 // XML Syntax Colouring
         index = 0;
         while(type.equals("xml") && (found = text.indexOf("<", index)) != -1) {

                 // Set normal color
		 document.setCharacterAttributes(index, found - index + 1, setNormal, false);
		 
		 int end = text.indexOf(">", found + 1);
		 if(end != -1) {
			 document.setCharacterAttributes(found, end - found + 1, setXml, false);
			 index = end + 1;
		 }
		 else {
			 index = found + 1;
		 }
	 }

     
	 // Text File Syntax Colouring
 	 index = 0;
         while(type.equals("text") && (found = text.indexOf("#", index)) != -1) {
		// Set normal color
		 document.setCharacterAttributes(index, found - index + 1, setNormal, false);
		 int end = text.indexOf("\n", found + 1);
		 if(end != -1) {
			 document.setCharacterAttributes(found, end - found + 1, setText, false);
			 index = end + 1;
		 }
		 else {
			 index = found + 1;
		 }
	 }

	 // String ("...") Syntax Colouring
	 index = 0;
         while((found = text.indexOf("\"", index)) != -1) {

	
		 
		 int end = text.indexOf("\"", found + 1);
		 if(end != -1) {
			 document.setCharacterAttributes(found, end - found + 1, setString, false);
			 index = end + 1;
		 }
		 else {
			 index = found + 1;
		 }
	 }
	

	 
    }
}
