/*
 * XmlFileCreator.java
 *
 * Created on 14. August 2001, 08:48
 */

package org.paw.gui;


import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;


/**
 *
 * @author  LUELLJOC
 * @version 
 */
public class XmlFileCreator {

    private AdminConnection connection;
    
    /** Creates new XmlFileCreator */
    public XmlFileCreator(AdminConnection connection) {
        this.connection = connection;
    }
    
    /** Creates new XmlFileCreator */
    public XmlFileCreator() {
        this.connection = null;
    }
    
    public String getServerConfig() {
        GuiServerConfig serverConf = connection.getServerConfig();
    
        GuiAdminConfig adminConf = connection.getAdminConfig();
        
        // Build XML document
        Element root = new Element("server");
        
        Element port = new Element("port").setText(serverConf.getPort());
        Element log = new Element("log").setText(serverConf.getLog());

        root.addContent(port);
        root.addContent(log);
        
        if (serverConf.useProxy()) {
            root.addContent(new Comment("Proxy settings"));
            Element proxyHost = 
                new Element("proxyhost").setText(serverConf.getProxyHost());
            Element proxyPort =
                new Element("proxyport").setText(serverConf.getProxyPort());

            root.addContent(proxyHost);
            root.addContent(proxyPort);
        }
        
        if (adminConf.isActive()) {
            root.addContent(new Comment("Admin Server settings"));
            Element admin = new Element("admin");
            Element adminPort =
                new Element("port").setText(adminConf.getPort());
            Element user =
                new Element("user").setText(adminConf.getUser());
            Element pass =
                new Element("pass").setText(adminConf.getPass());
            
            admin.addContent(adminPort);
            admin.addContent(user);
            admin.addContent(pass);
            root.addContent(admin);
        }
     
        // Prepare output
        Document doc = new Document(root);
        
        //System.out.println(prepareOutput(doc));
        
        return prepareOutput(doc);
    }

    public String getHandlerConfig() {
	Vector handlers = connection.getHandlers();
	return getHandlerConfig(handlers);
    }
    public String getHandlerConfig(Vector handlers) {

        // Build XML document
        Element root = new Element("handlers");
        
        for (Enumeration e = handlers.elements(); e.hasMoreElements();) {
            GuiHandler handler = (GuiHandler) e.nextElement();
            Element eHandler = new Element("handler");
            String active = handler.isActive() ? "active" : "inactive";

            eHandler.setAttribute(new Attribute("status", active));
            root.addContent(eHandler);
            
            Element name =
                new Element("name").setText(handler.getName());
            Element description =
                new Element("description").setText(handler.getDescription());
            
            String removableStr = handler.isRemovable() ? "true" : "false";
            Element removable =
                new Element("removable").setText(removableStr);
            Element id =
                new Element("id").setText(handler.getId());

            eHandler.addContent(name);
            eHandler.addContent(description);
            eHandler.addContent(removable);
            eHandler.addContent(id);
            
            // Get all files
            Element eFiles = new Element("files");

            eHandler.addContent(eFiles);
            
            Hashtable files = handler.getFiles();

            for (Enumeration f = files.keys(); f.hasMoreElements();) {
                String file = (String) f.nextElement();
                Element fileName =
                    new Element("file").setText(file);

                eFiles.addContent(fileName);
            }
            
            // Get all parameters
            Element eParams = new Element("params");

            eHandler.addContent(eParams);
            
            Vector params = handler.getParams();

            for (Enumeration p = params.elements(); p.hasMoreElements();) {
                GuiParam param = (GuiParam) p.nextElement();
                Element eParam =
                    new Element("param");

                eParam.setAttribute(new Attribute("name", param.getParam()));
                eParam.setAttribute(new Attribute("value", param.getValue()));
                eParams.addContent(eParam);
            }
            
        }
        
        // Prepare output
        Document doc = new Document(root);
        
        //System.out.println(prepareOutput(doc));
        
        return prepareOutput(doc);
    }
    public String getFilterConfig() {
    	Vector filters = connection.getFilters();
        return getFilterConfig(filters);
    }
    public String getFilterConfig(Vector filters) {
      
        // Build XML document
        Element root = new Element("filters");
        
        for (Enumeration e = filters.elements(); e.hasMoreElements();) {
            GuiFilter filter = (GuiFilter) e.nextElement();
            Element eFilter = new Element("filter");
            String active = filter.isActive() ? "active" : "inactive";

            eFilter.setAttribute(new Attribute("status", active));
            root.addContent(eFilter);
            
            Element name =
                new Element("name").setText(filter.getName());
            Element description =
                new Element("description").setText(filter.getDescription());
            
            String removableStr = filter.isRemovable() ? "true" : "false";
            Element removable =
                new Element("removable").setText(removableStr);
            Element id =
                new Element("id").setText(filter.getId());

            eFilter.addContent(name);
            eFilter.addContent(description);
            eFilter.addContent(removable);
            eFilter.addContent(id);
            
            // Get all files
            Element eFiles = new Element("files");

            eFilter.addContent(eFiles);
            
            Hashtable files = filter.getFiles();

            for (Enumeration f = files.keys(); f.hasMoreElements();) {
                String file = (String) f.nextElement();
                Element fileName =
                    new Element("file").setText(file);

                eFiles.addContent(fileName);
            }
            
            // Get all parameters
            Element eParams = new Element("params");

            eFilter.addContent(eParams);
            
            Vector params = filter.getParams();

            for (Enumeration p = params.elements(); p.hasMoreElements();) {
                GuiParam param = (GuiParam) p.nextElement();
                Element eParam =
                    new Element("param");

                eParam.setAttribute(new Attribute("name", param.getParam()));
                eParam.setAttribute(new Attribute("value", param.getValue()));
                eParams.addContent(eParam);
            }
            
        }
        
        // Prepare output
        Document doc = new Document(root);
        
        //System.out.println(prepareOutput(doc));
        
        return prepareOutput(doc);
    }

    public String getRegFilterConfig() throws CloneNotSupportedException {
        Vector repFilters = connection.getReplaceFilters();
        Vector sizeFilters = connection.getSizeFilters();
        Vector allFilters = (Vector) repFilters.clone();
	
	// Concat Vectors
        for (Enumeration cv = sizeFilters.elements(); cv.hasMoreElements();) {
            allFilters.add(cv.nextElement());
        }
	
	return getRegFilterConfig(allFilters);
    }
      

    public String getRegFilterConfig(Vector allFilters) throws CloneNotSupportedException {
        // Build XML document
        Element root = new Element("reg-filters");
   
        for (Enumeration e = allFilters.elements(); e.hasMoreElements();) {
            GuiRegExpFilter filter = (GuiRegExpFilter) e.nextElement();
            Element eFilter = new Element("reg-filter");

            eFilter.setAttribute(new Attribute("type", filter.getType()));
            String active = filter.isActive() ? "active" : "inactive";

            eFilter.setAttribute(new Attribute("status", active));
            root.addContent(eFilter);
            
            Element name =
                new Element("name").setText(filter.getName());
            Element description =
                new Element("description").setText(filter.getDescription());

            eFilter.addContent(name);
            eFilter.addContent(description);
            
            if (filter.getType().equals("size")) {
                Element file =
                    new Element("file").setText(filter.getFile());

                eFilter.addContent(file);
            }
            
            // Get all regular expressions
            
            Vector regs = filter.getRegExp();

            for (Enumeration r = regs.elements(); r.hasMoreElements();) {
                GuiRegExp reg = (GuiRegExp) r.nextElement();
                
                Element eReg = new Element("regexp");

                eFilter.addContent(eReg);
                Element match =
                    new Element("match").setText(reg.getMatch());
                Element subst =
                    new Element("subst").setText(reg.getSubst());

                eReg.addContent(match);
                eReg.addContent(subst);
            }
        }

        // Prepare output
        Document doc = new Document(root);
        
        //System.out.println(prepareOutput(doc));
        
        return prepareOutput(doc);
    }
        
    private String prepareOutput(Document doc) {
        XMLOutputter xmlout = new XMLOutputter();

        xmlout.setIndent(true); 
        xmlout.setNewlines(true);
        //xmlout.setTextNormalize(true);
        xmlout.setEncoding("ISO-8859-1");
        
        String xml = xmlout.outputString(doc);
        
        return xml;
    }
}
