package org.paw.handler;

import sunlabs.brazil.handler.ResourceHandler;
import sunlabs.brazil.server.FileHandler;
import sunlabs.brazil.server.Handler;
import sunlabs.brazil.server.Request;
import sunlabs.brazil.server.Server;
import sunlabs.brazil.util.http.HttpInputStream;
import sunlabs.brazil.util.regexp.Regexp;
import sunlabs.brazil.util.http.MimeHeaders;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;
import java.util.Hashtable;
import java.util.Enumeration;

import gnu.regexp.*;


public class RedirectHandler
    implements Handler
{
    private static final String URLS = "rules";

    String prefix;

    Hashtable urlsHash = null;

    public boolean
    init(Server server, String prefix)
    {
	this.prefix = prefix;

	Properties props = server.props;
	
	try {
	    String urls = props.getProperty(prefix + URLS);
            // Create headers hash
            urlsHash = loadUrls(props, prefix, urls);
	} catch (Exception e) {
	    server.log(Server.LOG_DIAGNOSTIC, prefix,
		    "error in headers: " + e.getMessage());
	}
	return true;
    }

    public boolean
    respond(Request request)
	throws IOException
    {
        MimeHeaders headers = request.headers;

        String fullUrl = request.url;

        if(request.query.length() > 0) {
           fullUrl += "?" + request.query;
        }

        String fullUrlOrg = fullUrl;


        for(Enumeration enum = urlsHash.keys(); enum.hasMoreElements();) {
            String org =    (String) enum.nextElement(); 
            String subst  = (String) urlsHash.get(org);
 
            try {
               RE re = new RE(org);
               fullUrl = re.substitute(fullUrl, subst);
            }
            catch(REException ex) {
               ex.printStackTrace();
            }

            // Leave loop after first match
            if(!fullUrl.equals(fullUrlOrg)) {
               break;
            }
        }


        if(!fullUrl.equals(fullUrlOrg)) {
           request.redirect(fullUrl, null);
	   return true;
        }

        return false;
         
    }

    public Hashtable
    loadUrls(Properties props, String prefix, String file)
    {
	Hashtable ht = new Hashtable();
	try {
	    HttpInputStream in = new HttpInputStream(
		    ResourceHandler.getResourceStream(props, prefix, file));

	    String line;
	    while ((line = in.readLine()) != null) {


		if ((line.length() == 0) || (line.charAt(0) == '#')) {
		    continue;
		}

                String url = line.substring(0, line.indexOf("\t"));
                String value = line.substring(line.lastIndexOf("\t") + 1);
                ht.put(url, value);
	    }

	    return ht;
	} catch (Exception e) {
            e.printStackTrace();
            // Return a valid hashtable in any case
	    return ht;
	}
    }

}
