package org.paw.server;


import java.io.*;
import java.util.*;
import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;
import sunlabs.brazil.util.Format; 
import sunlabs.brazil.server.Server; 
import java.util.Properties; 
import java.net.ServerSocket; 
import java.io.InputStream;
import java.net.BindException;


public class PawServer {
    String serverConfig = null;
    int queue = 1024;
    int port = 8080;

    String serverPort = null;
    String log = null;
    String proxyHost = null;
    String proxyPort = null;

    boolean adminActive = false;
    String adminPort = null;
    String adminUser = null;
    String adminPass = null;

    Server server = null;
    Properties config = null;
  
    boolean serverStarted = false;
  
    private Vector handlers = new Vector();
    private Vector filters = new Vector();
    
    public static final String LOG_FILE = "logs/server.log";

    public PawServer(String serverConfig) {
        PrintStream out = System.out;
	
	this.redirectStdout();

        this.serverConfig = serverConfig;
        init();
        serverStarted = startServer();

        //if(serverStarted)
        //out.println("PAW Server started on port " + port);

    }

    public void init() {
        config = new Properties();
        handlers = new Vector();
        filters = new Vector();

        addMimetypes();

        readServerConfig();
        addProperties();
    }

    public boolean startServer() {
        PrintStream out = System.out;

        port = new Integer(serverPort).intValue();

        String handler = config.getProperty("handler");

        server = new Server();
        server.logLevel = Integer.decode(log).intValue();

        try {
            ServerSocket listen = new ServerSocket(port, queue);

            server.setup(listen, handler, config);
            server.start();
        } catch (BindException e) {
            out.println("Port " + port + " is already in use");
            return false;
        } catch (IOException ie) {
            out.println(ie.getMessage());
            return false;
        }

        out.println("PAW Server started on port " + port);
        return true;
    }

    public void readServerConfig() {

        proxyHost = null;
        proxyPort = null;
        try {
            // No validation
            SAXBuilder builder = new SAXBuilder(false);
            Document doc = builder.build(new File(serverConfig));

            // Get root element
            Element root = doc.getRootElement();

            serverPort = root.getChild("port").getText();
  
            log = root.getChild("log").getText();

            if (root.getChild("proxyhost") != null)
                proxyHost = root.getChild("proxyhost").getText();

            if (root.getChild("proxyport") != null)
                proxyPort = root.getChild("proxyport").getText();

            // Read admin settings
            adminActive = false;
            adminPort = null;
            adminUser = null;
            adminPass = null;
       
            Element admin = root.getChild("admin");

            if (admin != null) {
                adminActive = true;
                adminPort = admin.getChild("port").getText();
                adminUser = admin.getChild("user").getText();
                adminPass = admin.getChild("pass").getText();
          
            }

        } catch (JDOMException jdome) {
            System.out.println(jdome.getMessage());
        }
    }

    public void addProperties() {   
        getHandlers("conf/handler.xml");
        String useHandlers = "";
    
        // Set some defaults
        this.config.put("port", serverPort);
        this.config.put("handler", "main");
        this.config.put("root", ".");
        this.config.put("main.class", "sunlabs.brazil.server.ChainHandler");
    
        // Proxy handler must be there
    
        this.config.put("proxy.class", "sunlabs.brazil.filter.FilterHandler");
        this.config.put("proxy.prefix", "");
        this.config.put("proxy.handler", "sunlabs.brazil.proxy.ProxyHandler");
    
        for (Enumeration e = handlers.elements(); e.hasMoreElements();) {
            PawHandler ph = (PawHandler) e.nextElement();

            useHandlers += " " + ph.getId();
        
            for (Enumeration ep = ph.getParams().elements(); ep.hasMoreElements();) {
                PawParam pp = (PawParam) ep.nextElement();

                //System.out.println(pp.getParam() + "=" + pp.getValue());
                this.config.put(pp.getParam(), pp.getValue());
            }
        }
    
        // Proxy handler should be the last in line
        useHandlers += " proxy";
    
        this.config.put("main.handlers", useHandlers);
    
        // Now handle the filters

        String useFilters;

        getFilters("conf/filter.xml");

        // If there are filters use UnPack filter as first filter in line.
        // Othterwise use only the standard filter
        if(!filters.isEmpty()) {
          useFilters = "unpack";
          this.config.put("unpack.class", "org.paw.filter.UnpackFilter");
        } else {
          useFilters = "standard";
          this.config.put("standard.class", "org.paw.filter.StandardFilter");
        }

    
        for (Enumeration ef = filters.elements(); ef.hasMoreElements();) {
            PawFilter pf = (PawFilter) ef.nextElement();

            useFilters += " " + pf.getId();
        
            for (Enumeration epf = pf.getParams().elements(); epf.hasMoreElements();) {
                PawParam pp = (PawParam) epf.nextElement(); 

                this.config.put(pp.getParam(), pp.getValue());
            }
        }

        if(!filters.isEmpty()) {
            useFilters += " repack";
            this.config.put("repack.class", "org.paw.filter.RepackFilter");
        }
 
        this.config.put("proxy.filters", useFilters);

        /*
         this.config.put("paw.class", "org.paw.server.PawRegFilter");
         **/
        this.config.put("paw.config", "conf/reg-filter.xml");

        // Additional Proxy settings
        if (proxyHost != null && proxyPort != null) {
            this.config.put("proxy.handler.proxyHost", proxyHost);
            this.config.put("proxy.handler.proxyPort", proxyPort);
        }
    
    }

    public void addMimetypes() {
        this.config.put("mime.html", "text/html"); 
        this.config.put("mime.txt", "text/plain"); 
        this.config.put("mime.gif", "image/gif"); 
        this.config.put("mime.jpg", "image/jpeg"); 
        this.config.put("mime.css", "text/x-css-stylesheet"); 
        this.config.put("mime.class", "application/octet-stream"); 
        this.config.put("mime.jar", "application/octet-stream");
        this.config.put("mime.jib", "application/octet-stream");
    }

    private boolean getHandlers(String config) {
        try {
            // No validation
            SAXBuilder builder = new SAXBuilder(false);
            Document doc = builder.build(new File(config));

            // Get root element
            Element root = doc.getRootElement();

            List handlerList = root.getChildren("handler");

            // Loop through all filters
            Iterator i = handlerList.iterator();

            while (i.hasNext()) {
                Element handler = (Element) i.next();
        
                PawHandler ph = new PawHandler();
        
                String status = handler.getAttributeValue("status");
                String name = handler.getChild("name").getText();
                String description = handler.getChild("description").getText();
                String removable = handler.getChild("removable").getText();
                String id = handler.getChild("id").getText();
        
                Element files = handler.getChild("files");

                if (files != null) {
                    List fileList = files.getChildren("file");
        
                    Iterator fe = fileList.iterator();

                    while (fe.hasNext()) {
                        Element fileEntry = (Element) fe.next();
                        String filename = fileEntry.getText();

                        ph.addFile(filename);
                    }
                }
        
                Element params = handler.getChild("params");

                if (params != null) {
                    List paramList = params.getChildren("param");
                    Iterator pe = paramList.iterator();
        
                    while (pe.hasNext()) {    
                        Element paramEntry = (Element) pe.next();
         
                        String param = paramEntry.getAttributeValue("name");
                        String value = paramEntry.getAttributeValue("value");
      
                        ph.addParam(param, value);
                    }
                }

                ph.setStatus(status);
                ph.setName(name);
                ph.setDescription(description);
        
                if (removable.equals("true"))
                    ph.setRemovable(true);
                else
                    ph.setRemovable(false);
        
                ph.setId(id);
        
                if (ph.getStatus().equals("active")) {
                    handlers.add(ph);      
                    //System.out.println("Added handler " + name);
                }
            }
        } catch (Exception ex) {
            //System.err.println(ex.getMessage());
            ex.printStackTrace();
            return false;
        }
    
        return true;
    }
  
    private boolean getFilters(String config) {
        try {
            // No validation
            SAXBuilder builder = new SAXBuilder(false);
            Document doc = builder.build(new File(config));

            // Get root element
            Element root = doc.getRootElement();

            List filterList = root.getChildren("filter");

            // Loop through all filters
            Iterator i = filterList.iterator();

            while (i.hasNext()) {
                Element filter = (Element) i.next();
        
                PawFilter pf = new PawFilter();
        
                String status = filter.getAttributeValue("status");
                String name = filter.getChild("name").getText();
                String description = filter.getChild("description").getText();
                String removable = filter.getChild("removable").getText();
                String id = filter.getChild("id").getText();
        
                Element files = filter.getChild("files");
                List fileList = filter.getChildren("file");
        
                Iterator fe = fileList.iterator();

                while (fe.hasNext()) {
                    Element fileEntry = (Element) fe.next();
                    String filename = fileEntry.getText();

                    pf.addFile(filename);
                }
        
                Element params = filter.getChild("params");
                List paramList = params.getChildren("param");
                Iterator pe = paramList.iterator();
        
                while (pe.hasNext()) {    
                    Element paramEntry = (Element) pe.next();
         
                    String param = paramEntry.getAttributeValue("name");
                    String value = paramEntry.getAttributeValue("value");
      
                    pf.addParam(param, value);
                }

                pf.setStatus(status);
                pf.setName(name);
                pf.setDescription(description);
        
                if (removable.equals("true"))
                    pf.setRemovable(true);
                else
                    pf.setRemovable(false);
        
                pf.setId(id);
        
                if (pf.getStatus().equals("active")) {
                    filters.add(pf);      
                    //System.out.println("Added handler " + name);
                }
            }
        } catch (Exception ex) {
            //System.err.println(ex.getMessage());
            ex.printStackTrace();
            return false;
        }
    
        return true;
    }

    public  void redirectStdout() {
       // Redirect Stdout to logfile
       try {
            File logFile = new File(LOG_FILE);

            // Delete logfile if exists
            if (logFile.exists())
                logFile.delete();

            FileOutputStream logFos = new FileOutputStream(logFile);
            System.setOut(new PrintStream(logFos));

	    // Initialize logfile
	    System.out.println("# Start of PAW logfile");
	    System.out.println("# " + new Date());

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
