/*
 * VirtualHostHandler.java
 *
 * Brazil project web application toolkit,
 * export version: 2.0 
 * Copyright (c) 1999-2002 Sun Microsystems, Inc.
 *
 * Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version 
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is included as the file "license.terms",
 * and also available at http://www.sun.com/
 * 
 * The Original Code is from:
 *    Brazil project web application toolkit release 2.0.
 * The Initial Developer of the Original Code is: suhler.
 * Portions created by suhler are Copyright (C) Sun Microsystems, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s): cstevens, suhler.
 *
 * Version:  1.5
 * Created by suhler on 99/06/28
 * Last modified by suhler on 02/07/24 10:44:37
 */

package sunlabs.brazil.handler;

import sunlabs.brazil.server.Handler;
import sunlabs.brazil.server.Request;
import sunlabs.brazil.server.Server;
import java.io.IOException;

/**
 * Handler for managing virtual hosts.
 * This prefixes the host name (from the host:) header onto the
 * url and passes the request along.  It could be more useful, but isn't.
 * If no host is provided, the host "default" is used instead.
 * <br>
 * For more sophisticated virtual host handling, use the
 * {@link MultiHostHandler} instead.
 *
 * @author		Stephen Uhler
 * @version		1.5, 02/07/24
 */

public class VirtualHostHandler implements Handler {

    public boolean
    init(Server server, String prefix) {
	server.log(Server.LOG_DIAGNOSTIC, prefix,
	    "Starting virtual host handler");
	return true;
    }

    /**
     * look for host header, tack on front of url.
     */

    public boolean
    respond(Request request) throws IOException {
    	String host = (String) request.headers.get("Host");
    	if (host == null) {
    	    host = "default";
	}
	try {
	    host = host.substring(0,host.indexOf(":"));
	} catch (Exception e) {}
	request.log(Server.LOG_DIAGNOSTIC, host +
		" added to url " + request.url);
	request.url = "/" + host.toLowerCase() + request.url;
	return false;
    }
}
