/*
 * ExprPropsHandler.java
 *
 * Brazil project web application toolkit,
 * export version: 2.0 
 * Copyright (c) 2001-2002 Sun Microsystems, Inc.
 *
 * Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version 
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is included as the file "license.terms",
 * and also available at http://www.sun.com/
 * 
 * The Original Code is from:
 *    Brazil project web application toolkit release 2.0.
 * The Initial Developer of the Original Code is: drach.
 * Portions created by drach are Copyright (C) Sun Microsystems, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s): drach, suhler.
 *
 * Version:  1.14
 * Created by drach on 01/07/11
 * Last modified by drach on 02/05/02 13:09:16
 */

package sunlabs.brazil.properties;

import sunlabs.brazil.server.Handler;
import sunlabs.brazil.server.Request;
import sunlabs.brazil.server.Server;
import sunlabs.brazil.handler.MatchString;

import java.io.IOException;

/**
 *
 * <code>ExprPropsHandler</code> creates an <code>ExprProps</code>
 * instance and adds it to the properties list maintained by the
 * <code>Request</code> object.
 * <p>
 * The following configuration parameter is used:
 * <dl>
 * <dt>prefix
 * <dd>Only URL's that start with this are allowed.
 * (See {@link MatchString}).
 * </dl>
 * 
 * @author	Steve Drach &lt;drach@sun.com&gt;
 * @version     1.14, 02/05/02
 *
 * @see ExprProps
 * @see sunlabs.brazil.server.Request
 */
public class ExprPropsHandler implements Handler {

    private MatchString isMine;            // check for matching url

    public boolean
    init(Server server, String prefix) {
	isMine = new MatchString(prefix, server.props);
	return true;
    }

    /**
     *
     * Creates an instance of <code>ExprProps</code> that uses
     * <code>request.props</code> for the wrapped
     * <code>Calculator</code>'s symbol table.
     *
     * @see sunlabs.brazil.util.Calculator
     *
     * @return false
     */
    public boolean
    respond(Request request) throws IOException {
	if (isMine.match(request.url)) {
	    PropertiesList pl = new PropertiesList(new ExprProps(request), true);
	    pl.addBefore(request.serverProps);
	}
	return false;
    }
}
